@@ -52,16 +52,14 @@
 #include "handicaps.h"
 
 /* ai/default */
-#include "aidata.h"
 #include "aihand.h"
-#include "ailog.h"
-#include "aiplayer.h"
 #include "aitools.h"
-#include "aiunit.h"
+#include "daidata.h"
 #include "daidiplomacy.h"
 #include "daidomestic.h"
 #include "daimilitary.h"
 #include "daieffects.h"
+#include "daiplayer.h"
 #include "daisettler.h"
 
 #include "daicity.h"
@@ -389,8 +387,8 @@ static void dai_upgrade_units(struct city *pcity, int limit, bool military)
   {
     /* Find upgrade unit actions */
     int i = 0;
-    action_list_add_all_by_result(upgrade_actions, &i, ACTRES_UPGRADE_UNIT);
-    action_list_end(upgrade_actions, i);
+    action_array_add_all_by_result(upgrade_actions, &i, ACTRES_UPGRADE_UNIT);
+    action_array_end(upgrade_actions, i);
   }
 
   dai_calc_data(pplayer, NULL, &expenses, NULL);
@@ -608,7 +606,7 @@ static void dai_spend_gold(struct ai_type *ait, struct player *pplayer)
 
     if (is_unit_choice_type(bestchoice.type)
         && utype_is_cityfounder(bestchoice.value.utype)) {
-      if (get_city_bonus(pcity, EFT_GROWTH_FOOD) <= 0
+      if (get_city_bonus(pcity, EFT_SHRINK_FOOD) <= 0
           && bestchoice.value.utype->pop_cost > 0
           && city_size_get(pcity) <= bestchoice.value.utype->pop_cost) {
         /* Don't buy settlers in cities that cannot afford the population cost. */
@@ -724,8 +722,8 @@ static int unit_foodbox_cost(struct unit *punit)
     int cost = 0;
     int i;
 
-    /* The default is to lose 100%. The growth bonus reduces this. */
-    int foodloss_pct = 100 - city_granary_savings(pcity);
+    /* The default is to lose 100%. The granary bonus reduces this. */
+    int foodloss_pct = 100 - city_shrink_granary_savings(pcity);
 
     foodloss_pct = CLIP(0, foodloss_pct, 100);
     fc_assert_ret_val(pcity != NULL, -1);
@@ -1259,6 +1257,7 @@ static int action_target_neg_util(action_id act_id,
   case ACTRES_ESTABLISH_EMBASSY:
   case ACTRES_SPY_INVESTIGATE_CITY:
   case ACTRES_MARKETPLACE:
+  case ACTRES_SPY_ESCAPE:
     /* TODO: Individual and well balanced values */
     return 1;
 
@@ -1291,31 +1290,37 @@ static int action_target_neg_util(action_id act_id,
   case ACTRES_NUKE_UNITS:
   case ACTRES_PARADROP:
   case ACTRES_ATTACK:
+  case ACTRES_WIPE_UNITS:
+  case ACTRES_COLLECT_RANSOM:
   case ACTRES_HEAL_UNIT:
   case ACTRES_TRANSFORM_TERRAIN:
   case ACTRES_CULTIVATE:
   case ACTRES_PLANT:
   case ACTRES_PILLAGE:
-  case ACTRES_CLEAN_POLLUTION:
-  case ACTRES_CLEAN_FALLOUT:
+  case ACTRES_CLEAN:
   case ACTRES_FORTIFY:
   case ACTRES_ROAD:
   case ACTRES_CONVERT:
   case ACTRES_BASE:
   case ACTRES_MINE:
   case ACTRES_IRRIGATE:
-  case ACTRES_TRANSPORT_ALIGHT:
+  case ACTRES_TRANSPORT_DEBOARD:
   case ACTRES_TRANSPORT_BOARD:
   case ACTRES_TRANSPORT_UNLOAD:
+  case ACTRES_TRANSPORT_LOAD:
   case ACTRES_TRANSPORT_DISEMBARK:
   case ACTRES_TRANSPORT_EMBARK:
   case ACTRES_CONQUER_EXTRAS:
   case ACTRES_HUT_ENTER:
   case ACTRES_HUT_FRIGHTEN:
   case ACTRES_UNIT_MOVE:
+  case ACTRES_TELEPORT:
+  case ACTRES_TELEPORT_CONQUER:
   case ACTRES_HOMELESS:
+  case ACTRES_ENABLER_CHECK:
     fc_assert_msg(action_id_get_target_kind(act_id) == ATK_CITY,
                   "Action not aimed at cities");
+    break;
   }
 
   /* Wrong action. Ignore it. */
@@ -1511,8 +1516,8 @@ static adv_want base_want(struct ai_type *ait, struct player *pplayer,
 
   if (is_wonder(pimprove)) {
     if (is_great_wonder(pimprove)) {
-      wonder_player_id =
-          game.info.great_wonder_owners[improvement_index(pimprove)];
+      wonder_player_id
+        = game.info.great_wonder_owners[improvement_index(pimprove)];
     }
     wonder_city_id = pplayer->wonders[improvement_index(pimprove)];
   }
@@ -1585,7 +1590,7 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
     .kind = VUT_IMPROVEMENT,
     .value = {.building = pimprove}
   };
-  const bool is_coinage = improvement_has_flag(pimprove, IF_GOLD);
+  const bool is_convert = is_convert_improvement(pimprove);
   int turns = 9999;
   int place = tile_continent(pcity->tile);
 
@@ -1618,14 +1623,17 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
     }
   } players_iterate_end;
 
-  if (is_coinage) {
-    /* Since coinage contains some entirely spurious ruleset values,
-     * we need to hard-code a sensible want.
-     * We must otherwise handle the special IF_GOLD improvement
-     * like the others, so the AI will research techs that make it available,
+  if (is_convert) {
+    /* Since coinage-like improvements contains some entirely spurious
+     * ruleset values, we need to hard-code a sensible want.
+     * We must otherwise handle IG_CONVERT improvements like the others,
+     * so the AI will research techs that make it available,
      * for rulesets that do not provide it from the start.
      */
-    v += TRADE_WEIGHTING / 10;
+    if (improvement_has_flag(pimprove, IF_GOLD)) {
+      v += TRADE_WEIGHTING / 10;
+    }
+    /* Without relevant flags, base want remains 0. */
   } else {
     /* Base want is calculated above using a more direct approach. */
     v += base_want(ait, pplayer, pcity, pimprove);
@@ -1637,7 +1645,7 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
     }
   }
 
-  if (!is_coinage) {
+  if (!is_convert) {
     /* Adjust by building cost */
     /* FIXME: ought to reduce by upkeep cost and amortise by building cost */
     v -= (impr_build_shield_cost(pcity, pimprove)
@@ -1662,7 +1670,8 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
   cities[REQ_RANGE_CITY] = cities[REQ_RANGE_LOCAL] = 1;
 
   /* Invalid building range */
-  cities[REQ_RANGE_ADJACENT] = cities[REQ_RANGE_CADJACENT] = 0;
+  cities[REQ_RANGE_ADJACENT] = cities[REQ_RANGE_CADJACENT]
+    = cities[REQ_RANGE_TILE] = 0;
 
   players_iterate(aplayer) {
     int potential = (aplayer->server.bulbs_last_turn
@@ -1693,8 +1702,7 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
     requirement_vector_iterate(&peffect->reqs, preq) {
       /* Check if all the requirements for the currently evaluated effect
        * are met, except for having the building that we are evaluating. */
-      if (VUT_IMPROVEMENT == preq->source.kind
-          && preq->source.value.building == pimprove) {
+      if (universal_fulfills_requirement(preq, &source) == ITF_YES) {
         if (preq->range < range) {
           /* More limited range */
           range = preq->range;
@@ -1702,9 +1710,10 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
         present = preq->present;
         continue;
       }
+
       if (!is_req_active(&effect_ctxt, NULL, preq, RPT_POSSIBLE)) {
-	active = FALSE;
-	if (VUT_ADVANCE == preq->source.kind && preq->present) {
+        active = FALSE;
+        if (VUT_ADVANCE == preq->source.kind && preq->present) {
           /* This missing requirement is a missing tech requirement.
            * This will be for some additional effect
            * (For example, in the default ruleset, Mysticism increases
@@ -1735,7 +1744,7 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
 	v += v1;
       } else if (v1 > 0) {
         /* If value of the effect is negative, do not hold it against
-         * the tech - having the tech wont force one to build the
+         * the tech - having the tech won't force one to build the
          * building. */
 
 	/* We might want the technology that will enable this
@@ -1783,8 +1792,7 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
       enum req_range range = REQ_RANGE_LOCAL;
 
       requirement_vector_iterate(&(enabler->target_reqs), preq) {
-        if (VUT_IMPROVEMENT == preq->source.kind
-            && preq->source.value.building == pimprove) {
+        if (universal_fulfills_requirement(preq, &source) == ITF_YES) {
           /* Pretend the building is there */
           if (preq->present) {
             range = preq->range; /* Assumption: Max one pr vector */
@@ -1850,8 +1858,8 @@ static void adjust_improvement_wants_by_effects(struct ai_type *ait,
     can_build = can_build && all_met;
   }
 
-  if (is_coinage && can_build) {
-    /* Could have a negative want for coinage,
+  if (is_convert && can_build) {
+    /* Could have a negative want for coinage-like improvements,
      * if we have some stock in a building already. */
     pcity->server.adv->building_want[improvement_index(pimprove)] += v;
   } else if (!already && can_build) {
@@ -1909,7 +1917,7 @@ static bool should_force_recalc(struct city *pcity)
 {
   return city_built_last_turn(pcity)
       || (VUT_IMPROVEMENT == pcity->production.kind
-       && !improvement_has_flag(pcity->production.value.building, IF_GOLD)
+       && !is_convert_improvement(pcity->production.value.building)
        && !dai_can_city_build_improvement_later
            (pcity, pcity->production.value.building));
 }
@@ -1963,11 +1971,7 @@ void dai_build_adv_adjust(struct ai_type *ait, struct player *pplayer,
   } city_list_iterate_end;
 
   improvement_iterate(pimprove) {
-    const bool is_coinage = improvement_has_flag(pimprove, IF_GOLD);
-
-    /* Handle coinage specially because you can never complete coinage */
-    if (is_coinage
-        || can_player_build_improvement_later(pplayer, pimprove)) {
+    if (can_player_build_improvement_later(pplayer, pimprove)) {
       city_list_iterate(pplayer->cities, pcity) {
         struct ai_city *city_data = def_ai_city_data(pcity, ait);
 
@@ -2073,6 +2077,7 @@ static bool dai_cant_help_req(const struct req_context *context,
   case VUT_TERRAINCLASS:
   case VUT_TERRFLAG:
   case VUT_TERRAINALTER:
+  case VUT_CITYTILE:
     return !is_req_active(context, NULL, req, RPT_POSSIBLE);
   default:
     return is_req_preventing(context, NULL, req, RPT_POSSIBLE) > REQUCH_NO;
@@ -2097,7 +2102,6 @@ bool dai_can_city_build_improvement_later(const struct city *pcity,
                                               pimprove)) {
     return FALSE;
   }
-
   /* Check for requirements that aren't met and that are unchanging (so
    * they can never be met). */
   requirement_vector_iterate(&pimprove->reqs, preq) {
@@ -2172,10 +2176,27 @@ Impr_type_id dai_find_source_building(struct city *pcity,
           building = preq->source.value.building;
 
           if (!can_city_build_improvement_now(pcity, building)
-              || !is_improvement(building)) {          
+              || !is_improvement(building)) {
             building = NULL;
             break;
           }
+        } else if (VUT_IMPR_FLAG == preq->source.kind && preq->present) {
+          /* TODO: Ruleset cache for buildings with specific flag */
+          improvement_iterate(impr) {
+            if (improvement_has_flag(impr, preq->source.value.impr_flag)) {
+              if (can_city_build_improvement_now(pcity, impr)
+                  && is_improvement(impr)) {
+                if (building == NULL) {
+                  building = impr;
+                  break;
+                }
+              }
+            }
+          } improvement_iterate_end;
+
+          if (building == NULL) {
+            break;
+          }
         } else if (utype != NULL
                    && !is_req_active(&context, NULL, preq, RPT_POSSIBLE)) {
           /* Effect requires other kind of unit than what we are interested about */
