@@ -147,6 +147,7 @@ adv_want dai_effect_value(struct player *pplayer,
   int num;
   int trait;
   adv_want v = 0;
+  enum effect_type value_as;
 
   if (peffect->multiplier) {
     if (pplayer) {
@@ -168,16 +169,20 @@ adv_want dai_effect_value(struct player *pplayer,
     return 0;
   }
 
-  switch (peffect->type) {
+  value_as = user_effect_ai_valued_as(peffect->type);
+
+  switch (value_as) {
   /* These effects have already been evaluated in base_want() */
   case EFT_CAPITAL_CITY:
   case EFT_GOV_CENTER:
   case EFT_UPKEEP_FREE:
+  case EFT_IMPR_UPKEEP_REDUCTION:
   case EFT_TECH_UPKEEP_FREE:
   case EFT_POLLU_POP_PCT:
   case EFT_POLLU_POP_PCT_2:
   case EFT_POLLU_PROD_PCT:
   case EFT_OUTPUT_BONUS:
+  case EFT_OUTPUT_BONUS_ABSOLUTE:
   case EFT_OUTPUT_BONUS_2:
   case EFT_OUTPUT_ADD_TILE:
   case EFT_OUTPUT_INC_TILE:
@@ -186,6 +191,8 @@ adv_want dai_effect_value(struct player *pplayer,
   case EFT_OUTPUT_WASTE_BY_DISTANCE:
   case EFT_OUTPUT_WASTE_BY_REL_DISTANCE:
   case EFT_OUTPUT_WASTE_PCT:
+  case EFT_SURPLUS_WASTE_PCT:
+  case EFT_SURPLUS_WASTE_PCT_BY_REL_DISTANCE:
   case EFT_SPECIALIST_OUTPUT:
   case EFT_ENEMY_CITIZEN_UNHAPPY_PCT:
   case EFT_IRRIGATION_PCT:
@@ -245,6 +252,9 @@ adv_want dai_effect_value(struct player *pplayer,
       v += c * MAX(amount + 2, 1);
     }
     break;
+  case EFT_TECH_PARASITE_PCT_MAX:
+    amount = (amount * research_count()) / 100;
+    fc__fallthrough;
   case EFT_TECH_PARASITE:
     {
       int bulbs;
@@ -287,7 +297,10 @@ adv_want dai_effect_value(struct player *pplayer,
   case EFT_GROWTH_FOOD:
     /* FIXME: As total value is clipped 0 ... 100, single
      *        effect should have no value at all in some cases. */
-    v += c * 4 + (amount / 7) * pcity->surplus[O_FOOD];
+    v += c * 4 + (amount / 10) * MAX(pcity->surplus[O_FOOD], 0);
+    break;
+  case EFT_SHRINK_FOOD:
+    v += c * 4 + (amount / 15) * MAX(-pcity->surplus[O_FOOD], 1);
     break;
   case EFT_POPCOST_FREE:
     v += amount * city_granary_size(pcity->size) / 10;
@@ -408,7 +421,24 @@ adv_want dai_effect_value(struct player *pplayer,
         && (adv->dipl.spacerace_leader
             || adv->dipl.production_leader == pplayer
             || adv->dipl.tech_leader == pplayer)) {
-      v += 140;
+      int space_want;
+
+      if (pcity->id == adv->wonder_city) {
+        /* adjust_improvement_wants_by_effects() lowers want for any non-wonder
+         * target on wonder city. We want space part want reduced even more than
+         * regular buildings, as buildings are at least helping the specific city
+         * when finished. */
+        space_want = 120;
+      } else {
+        space_want = 210;
+      }
+
+      if (pplayer->spaceship.state == SSHIP_STARTED) {
+        /* We are already running. Let's not make it a half-hearted attempt. */
+        space_want *= 3;
+      }
+
+      v += space_want;
     }
     break;
   case EFT_SPY_RESISTANT:
@@ -419,6 +449,10 @@ adv_want dai_effect_value(struct player *pplayer,
     num = num_affected_units(peffect, adv);
     v += (8 * v * amount + num);
     break;
+  case EFT_ACTIVITY_TIME:
+    /* TODO: As an override, this can be either good or bad,
+     *       depending on the value being overridden. */
+    break;
   case EFT_UNIT_NO_LOSE_POP:
     v += unit_list_size(pcity->tile->units) * 2;
     break;
@@ -483,12 +517,12 @@ adv_want dai_effect_value(struct player *pplayer,
 
     if (affects_sea_capable_units) {
       if (is_ocean_tile(pcity->tile)) {
-        v += adv->threats.ocean[-tile_continent(pcity->tile)]
+        v += adv->oceans[-tile_continent(pcity->tile)].threat
           ? amount / 6 : amount / 25;
       } else {
         adjc_iterate(&(wld.map), pcity->tile, tile2) {
           if (is_ocean_tile(tile2)) {
-            if (adv->threats.ocean[-tile_continent(tile2)]) {
+            if (adv->oceans[-tile_continent(tile2)].threat) {
               v += amount / 6;
               break;
             }
@@ -496,18 +530,18 @@ adv_want dai_effect_value(struct player *pplayer,
         } adjc_iterate_end;
       }
     }
-    v += (amount / 25 + adv->threats.invasions - 1) * c; /* for wonder */
+    v += (amount / 25 + adv->threats.invasions - 1) * c; /* For wonder */
     if (capital || affects_land_capable_units) {
       Continent_id place = tile_continent(pcity->tile);
 
-      if ((place > 0 && adv->threats.continent[place])
+      if ((place > 0 && adv->continents[place].threat)
           || capital
           || (adv->threats.invasions
               /* FIXME: This ignores riverboats on some rulesets.
                         We should analyze rulesets when game starts
                         and have relevant checks here. */
               && is_terrain_class_near_tile(&(wld.map), pcity->tile, TC_OCEAN))) {
-        if (place > 0 && adv->threats.continent[place]) {
+        if (place > 0 && adv->continents[place].threat) {
           v += amount * 4 / 5;
         } else {
           v += amount / (!adv->threats.igwall ? (18 - capital * 6) : 18);
@@ -537,6 +571,10 @@ adv_want dai_effect_value(struct player *pplayer,
     /* This is based on average base upgrade price of 50. */
     v -= adv->stats.units.upgradeable * amount / 2;
     break;
+  case EFT_BORDER_STRENGTH_PCT:
+    v += amount / 4;
+    break;
+
   /* Currently not supported for building AI - wait for modpack users */
   case EFT_CITY_UNHAPPY_SIZE:
   case EFT_UNHAPPY_FACTOR:
@@ -567,10 +605,12 @@ adv_want dai_effect_value(struct player *pplayer,
   case EFT_CASUS_BELLI_SUCCESS:
   case EFT_CASUS_BELLI_COMPLETE:
   case EFT_ACTION_ODDS_PCT:
+  case EFT_ACTION_RESIST_PCT:
   case EFT_BORDER_VISION:
   case EFT_STEALINGS_IGNORE:
   case EFT_MAPS_STOLEN_PCT:
   case EFT_UNIT_SHIELD_VALUE_PCT:
+  case EFT_NUKE_BLAST_RADIUS_1_SQ:
   case EFT_HEAL_UNIT_PCT:
     break;
     /* This has no effect for AI */
@@ -621,17 +661,41 @@ adv_want dai_effect_value(struct player *pplayer,
     } iterate_outward_end;
     break;
   case EFT_MAX_TRADE_ROUTES:
-    trait = ai_trait_get_value(TRAIT_TRADER, pplayer);
-    v += amount
-      * (pow(2.0,
-             (double) get_city_bonus(pcity, EFT_TRADE_REVENUE_BONUS) / 1000.0)
-         + c)
-      * trait
-      / TRAIT_DEFAULT_VALUE;
-    if (city_num_trade_routes(pcity) >= max_trade_routes(pcity)
-        && amount > 0) {
-      /* Has no free trade routes before this */
-      v += trait;
+    {
+      /* We may know no caravans yet, then consider primary one
+       * (Supposed that if this effect is defined then some unit
+       * can establish trade routes) */
+      int trr = action_id_get_role(ACTION_TRADE_ROUTE);
+      const struct unit_type *van = best_role_unit(pcity, trr);
+      int bonus;
+
+      if (NULL == van){
+        if (0 < num_role_units(trr)) {
+          van = get_role_unit(trr, 0);
+        }
+      }
+      /* NOTE: if you have "DiplRel" in reqs, becomes 0 */
+      bonus =
+        get_target_bonus_effects(NULL,
+                                 &(const struct req_context) {
+                                   .player = city_owner(pcity),
+                                   .city = pcity,
+                                   .tile = city_tile(pcity),
+                                   .unittype = van
+                                 }, NULL, EFT_TRADE_REVENUE_BONUS);
+
+      trait = ai_trait_get_value(TRAIT_TRADER, pplayer);
+      v += amount
+        * (pow(2.0,
+               (double) bonus / 1000.0)
+           + c)
+        * trait
+        / TRAIT_DEFAULT_VALUE;
+      if (city_num_trade_routes(pcity) >= max_trade_routes(pcity)
+          && amount > 0) {
+        /* Has no free trade routes before this */
+        v += trait;
+      }
     }
     break;
   case EFT_TRADE_ROUTE_PCT:
@@ -724,15 +788,20 @@ bool dai_can_requirement_be_met_in_city(const struct requirement *preq,
     /* We can't meet a government requirement if we have a better one. */
     return !have_better_government(pplayer, preq->source.value.govern);
 
-  case VUT_IMPROVEMENT: {
+  case VUT_IMPROVEMENT:
+  case VUT_SITE:
+  {
     const struct impr_type *pimprove = preq->source.value.building;
 
-    if (preq->present && improvement_obsolete(pplayer, pimprove, pcity)) {
+    if (preq->present
+        && preq->source.kind == VUT_IMPROVEMENT
+        && improvement_obsolete(pplayer, pimprove, pcity)) {
       /* Would need to unobsolete a building, which is too hard. */
       return FALSE;
     } else if (!preq->present && pcity != NULL
                && I_NEVER < pcity->built[improvement_index(pimprove)].turn
-               && !can_improvement_go_obsolete(pimprove)) {
+               && (preq->source.kind != VUT_IMPROVEMENT
+                   || !can_improvement_go_obsolete(pimprove))) {
       /* Would need to unbuild an unobsoleteable building, which is too hard. */
       return FALSE;
     } else if (preq->present) {
@@ -760,16 +829,27 @@ bool dai_can_requirement_be_met_in_city(const struct requirement *preq,
     /* Crude, but the right answer needs to consider civil wars. */
     return nation_is_in_current_set(preq->source.value.nation);
 
-  case VUT_CITYSTATUS:
-    if (pcity == NULL || pcity->original == NULL) {
-      return preq->present;
-    }
-    if (preq->present) {
-      return city_owner(pcity) == pcity->original;
-    } else {
-      return city_owner(pcity) != pcity->original;
-    }
+  case VUT_ORIGINAL_OWNER:
+    return FALSE;
 
+  case VUT_CITYSTATUS:
+    if (pcity != NULL) {
+      if (preq->source.value.citystatus == CITYS_OWNED_BY_ORIGINAL
+          && pcity->original != NULL) {
+        if (preq->present) {
+          return city_owner(pcity) == pcity->original;
+        } else {
+          return city_owner(pcity) != pcity->original;
+        }
+      } else if (preq->source.value.citystatus == CITYS_TRANSFERRED) {
+        if ((preq->present && pcity->acquire_t == CACQ_FOUNDED)
+            || (!preq->present && pcity->acquire_t != CACQ_FOUNDED)) {
+          /* Would change only when the AI loses the city */
+          return FALSE;
+        }
+      }
+    }
+    break;
   case VUT_TERRAIN:
   case VUT_TERRAINCLASS:
   case VUT_TERRAINALTER:
@@ -787,6 +867,7 @@ bool dai_can_requirement_be_met_in_city(const struct requirement *preq,
   case VUT_MINSIZE:
   case VUT_MINYEAR:
   case VUT_TOPO:
+  case VUT_WRAP:
   case VUT_AGE:
   case VUT_TECHFLAG:
   case VUT_ACHIEVEMENT:
@@ -795,6 +876,10 @@ bool dai_can_requirement_be_met_in_city(const struct requirement *preq,
     /* No way to remove once present. */
     return preq->present;
 
+  case VUT_FORM_AGE:
+    /* FIXME: Sometimes it would be possible to convert back and forth */
+    return preq->present;
+
   case VUT_MINFOREIGNPCT:
     /* No way to add once lost. */
     return !preq->present;
@@ -803,16 +888,24 @@ bool dai_can_requirement_be_met_in_city(const struct requirement *preq,
   case VUT_NATIONGROUP:
   case VUT_AI_LEVEL:
   case VUT_SERVERSETTING:
+  case VUT_MINLATITUDE:
+  case VUT_MAXLATITUDE:
+  case VUT_PLAYER_FLAG:
+  case VUT_PLAYER_STATE:
     /* Beyond player control. */
     return FALSE;
 
   case VUT_OTYPE:
-  case VUT_CITYTILE:
   case VUT_IMPR_GENUS:
     /* Can always be achieved. */
     return TRUE;
 
+  case VUT_IMPR_FLAG:
+    /* TODO: Have at least some checks for this. */
+    break;
+
   case VUT_NONE:
+  case VUT_COUNTER:
   case VUT_UTYPE:
   case VUT_UTFLAG:
   case VUT_UCLASS:
@@ -826,6 +919,7 @@ bool dai_can_requirement_be_met_in_city(const struct requirement *preq,
   case VUT_STYLE:
   case VUT_UNITSTATE:
   case VUT_ACTIVITY:
+  case VUT_CITYTILE:
   case VUT_MINMOVES:
   case VUT_MINVETERAN:
   case VUT_MINHP:
@@ -836,5 +930,6 @@ bool dai_can_requirement_be_met_in_city(const struct requirement *preq,
     /* No sensible implementation possible with data available. */
     break;
   }
+
   return TRUE;
 }
