@@ -979,6 +979,9 @@ static void goto_fill_parameter_full(struct goto_map *goto_map,
     fc_assert_msg(hover_state != HOVER_GOTO_SEL_TGT,
                   "Path finding during target selection");
     break;
+  case HOVER_TELEPORT:
+    fc_assert_msg(hover_state != HOVER_TELEPORT, "Goto with HOVER_TELEPORT?");
+    break;
   case HOVER_NONE:
   case HOVER_PARADROP:
   case HOVER_ACT_SEL_TGT:
@@ -1367,8 +1370,7 @@ void request_orders_cleared(struct unit *punit)
 
   /* Clear the orders by sending an empty orders path. */
   log_goto_packet("Clearing orders for unit %d.", punit->id);
-  p.unit_id32 = punit->id;
-  p.unit_id16 = p.unit_id32;
+  p.unit_id = punit->id;
   p.src_tile = tile_index(unit_tile(punit));
   p.repeat = p.vigilant = FALSE;
   p.length = 0;
@@ -1481,8 +1483,7 @@ static void send_path_orders(struct unit *punit, struct pf_path *path,
   }
 
   memset(&p, 0, sizeof(p));
-  p.unit_id32 = punit->id;
-  p.unit_id16 = p.unit_id32;
+  p.unit_id = punit->id;
   p.src_tile = tile_index(unit_tile(punit));
   p.repeat = repeat;
   p.vigilant = vigilant;
@@ -1510,8 +1511,7 @@ static void send_rally_path_orders(struct city *pcity, struct unit *punit,
   struct packet_city_rally_point p;
 
   memset(&p, 0, sizeof(p));
-  p.city_id32 = pcity->id;
-  p.city_id16 = p.city_id32;
+  p.id = pcity->id;
   p.vigilant = vigilant;
   p.persistent = persistent;
 
@@ -1734,8 +1734,7 @@ void send_connect_route(enum unit_activity activity,
       i++;
     } while (i < goto_map->num_parts);
 
-    p.unit_id32 = punit->id;
-    p.unit_id16 = p.unit_id32;
+    p.unit_id = punit->id;
     p.src_tile = tile_index(unit_tile(punit));
     p.repeat = FALSE;
     p.vigilant = FALSE; /* Should be TRUE? */
