@@ -27,7 +27,7 @@
 /* client */
 #include "options.h"
 
-/* client/gui-gtk-4.0 */
+/* client/gui-gtk-3.22 */
 #include "colors.h"
 #include "dialogs.h"
 #include "gui_main.h"
@@ -67,8 +67,6 @@ enum {
   RESPONSE_SAVE
 };
 
-static GtkWidget *opt_popover = NULL;
-
 
 /* Option dialog main functions. */
 static struct option_dialog *
@@ -104,11 +102,11 @@ static void option_dialog_reponse_callback(GtkDialog *dialog,
 
   switch (response_id) {
   case RESPONSE_CANCEL:
-    gtk_window_destroy(GTK_WINDOW(dialog));
+    gtk_widget_destroy(GTK_WIDGET(dialog));
     break;
   case RESPONSE_OK:
     option_dialog_foreach(pdialog, option_dialog_option_apply);
-    gtk_window_destroy(GTK_WINDOW(dialog));
+    gtk_widget_destroy(GTK_WIDGET(dialog));
     break;
   case RESPONSE_APPLY:
     option_dialog_foreach(pdialog, option_dialog_option_apply);
@@ -120,8 +118,8 @@ static void option_dialog_reponse_callback(GtkDialog *dialog,
     option_dialog_foreach(pdialog, option_dialog_option_refresh);
     break;
   case RESPONSE_SAVE:
-    desired_settable_options_update();
-    options_save(NULL);
+    option_dialog_foreach(pdialog, option_dialog_option_apply);
+    queue_options_save(NULL);
     break;
   }
 }
@@ -132,11 +130,16 @@ static void option_dialog_reponse_callback(GtkDialog *dialog,
 static void option_dialog_destroy_callback(GtkWidget *object, gpointer data)
 {
   struct option_dialog *pdialog = (struct option_dialog *) data;
+  GtkWidget* cont_area;
+  GtkAllocation allocation;
+  int           baseline;
 
   /* Save size of the dialog. */
-  gtk_window_get_default_size(GTK_WINDOW(object),
-                              &opt_dlog_width,
-                              &opt_dlog_height);
+  cont_area = gtk_dialog_get_content_area (GTK_DIALOG(pdialog->shell));
+  gtk_widget_get_allocated_size(GTK_WIDGET(cont_area),
+                                &allocation, &baseline);
+  opt_dlog_width = allocation.width;
+  opt_dlog_height = allocation.height;
 
   if (NULL != pdialog->shell) {
     /* Mark as destroyed, see also option_dialog_destroy(). */
@@ -148,25 +151,20 @@ static void option_dialog_destroy_callback(GtkWidget *object, gpointer data)
 /************************************************************************//**
   Option refresh requested from menu.
 ****************************************************************************/
-static void option_refresh_callback(GSimpleAction *action, GVariant *parameter,
-                                    gpointer data)
+static void option_refresh_callback(GtkMenuItem *menuitem, gpointer data)
 {
-  struct option *poption = (struct option *)data;
+  struct option *poption = (struct option *) data;
   struct option_dialog *pdialog = option_dialog_get(option_optset(poption));
 
   if (NULL != pdialog) {
     option_dialog_option_refresh(poption);
   }
-
-  gtk_widget_unparent(opt_popover);
-  g_object_unref(opt_popover);
 }
 
 /************************************************************************//**
   Option reset requested from menu.
 ****************************************************************************/
-static void option_reset_callback(GSimpleAction *action, GVariant *parameter,
-                                  gpointer data)
+static void option_reset_callback(GtkMenuItem *menuitem, gpointer data)
 {
   struct option *poption = (struct option *) data;
   struct option_dialog *pdialog = option_dialog_get(option_optset(poption));
@@ -174,16 +172,12 @@ static void option_reset_callback(GSimpleAction *action, GVariant *parameter,
   if (NULL != pdialog) {
     option_dialog_option_reset(poption);
   }
-
-  gtk_widget_unparent(opt_popover);
-  g_object_unref(opt_popover);
 }
 
 /************************************************************************//**
   Option apply requested from menu.
 ****************************************************************************/
-static void option_apply_callback(GSimpleAction *action, GVariant *parameter,
-                                  gpointer data)
+static void option_apply_callback(GtkMenuItem *menuitem, gpointer data)
 {
   struct option *poption = (struct option *) data;
   struct option_dialog *pdialog = option_dialog_get(option_optset(poption));
@@ -191,62 +185,42 @@ static void option_apply_callback(GSimpleAction *action, GVariant *parameter,
   if (NULL != pdialog) {
     option_dialog_option_apply(poption);
   }
-
-  gtk_widget_unparent(opt_popover);
-  g_object_unref(opt_popover);
 }
 
 /************************************************************************//**
   Called when a button is pressed on an option.
 ****************************************************************************/
-static gboolean option_button_press_callback(GtkGestureClick *gesture,
-                                             int n_press,
-                                             double x, double y,
+static gboolean option_button_press_callback(GtkWidget *widget,
+                                             GdkEventButton *event,
                                              gpointer data)
 {
   struct option *poption = (struct option *) data;
-  GtkEventController *controller = GTK_EVENT_CONTROLLER(gesture);
-  GtkWidget *parent = gtk_event_controller_get_widget(controller);
-  GMenu *menu;
-  GActionGroup *group;
-  GSimpleAction *act;
-  GdkRectangle rect = { .x = x, .y = y, .width = 1, .height = 1};
+  GtkWidget *menu, *item;
 
-  if (!option_is_changeable(poption)) {
+  if (3 != event->button || !option_is_changeable(poption)) {
+    /* Only right button please! */
     return FALSE;
   }
 
-  group = G_ACTION_GROUP(g_simple_action_group_new());
+  menu = gtk_menu_new();
 
-  menu = g_menu_new();
+  item = gtk_menu_item_new_with_label(_("Refresh this option"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(option_refresh_callback), poption);
 
-  act = g_simple_action_new("refresh", NULL);
-  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
-  g_signal_connect(act, "activate", G_CALLBACK(option_refresh_callback), poption);
-  menu_item_append_unref(menu, g_menu_item_new(_("Refresh this option"),
-                                               "win.refresh"));
+  item = gtk_menu_item_new_with_label(_("Reset this option"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(option_reset_callback), poption);
 
-  act = g_simple_action_new("unit_reset", NULL);
-  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
-  g_signal_connect(act, "activate", G_CALLBACK(option_reset_callback), poption);
-  menu_item_append_unref(menu, g_menu_item_new(_("Reset this option"),
-                                               "win.unit_reset"));
+  item = gtk_menu_item_new_with_label(_("Apply the changes for this option"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(option_apply_callback), poption);
 
-  act = g_simple_action_new("units_apply", NULL);
-  g_action_map_add_action(G_ACTION_MAP(group), G_ACTION(act));
-  g_signal_connect(act, "activate", G_CALLBACK(option_apply_callback), poption);
-  menu_item_append_unref(menu,
-                         g_menu_item_new(_("Apply the changes for this option"),
-                                         "win.units_apply"));
-
-  opt_popover = gtk_popover_menu_new_from_model(G_MENU_MODEL(menu));
-  g_object_ref(opt_popover);
-  gtk_widget_insert_action_group(opt_popover, "win", group);
-
-  gtk_widget_set_parent(opt_popover, parent);
-  gtk_popover_set_pointing_to(GTK_POPOVER(opt_popover), &rect);
-
-  gtk_popover_popup(GTK_POPOVER(opt_popover));
+  gtk_widget_show_all(menu);
+  gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
 
   return TRUE;
 }
@@ -291,7 +265,9 @@ static void option_color_set_button_color(GtkButton *button,
   if (NULL == new_color) {
     if (NULL != current_color) {
       g_object_set_data(G_OBJECT(button), "color", NULL);
-      gtk_button_set_child(button, NULL);
+      if ((child = gtk_bin_get_child(GTK_BIN(button)))) {
+        gtk_widget_destroy(child);
+      }
     }
   } else {
     GdkPixbuf *pixbuf;
@@ -306,24 +282,24 @@ static void option_color_set_button_color(GtkButton *button,
       g_object_set_data_full(G_OBJECT(button), "color", current_color,
                              option_color_destroy_notify);
     }
-    gtk_button_set_child(button, NULL);
+    if ((child = gtk_bin_get_child(GTK_BIN(button)))) {
+      gtk_widget_destroy(child);
+    }
 
     /* Update the button. */
     {
       cairo_surface_t *surface = cairo_image_surface_create(
           CAIRO_FORMAT_RGB24, 16, 16);
       cairo_t *cr = cairo_create(surface);
-
       gdk_cairo_set_source_rgba(cr, current_color);
       cairo_paint(cr);
       cairo_destroy(cr);
       pixbuf = gdk_pixbuf_get_from_surface(surface, 0, 0, 16, 16);
       cairo_surface_destroy(surface);
     }
-
     child = gtk_image_new_from_pixbuf(pixbuf);
-    gtk_button_set_child(GTK_BUTTON(button), child);
-    gtk_widget_set_visible(child, TRUE);
+    gtk_container_add(GTK_CONTAINER(button), child);
+    gtk_widget_show(child);
     g_object_unref(G_OBJECT(pixbuf));
   }
 }
@@ -347,7 +323,7 @@ static void color_selector_response_callback(GtkDialog *dialog,
     option_color_set_button_color(GTK_BUTTON(data), &new_color);
   }
 
-  gtk_window_destroy(GTK_WINDOW(dialog));
+  gtk_widget_destroy(GTK_WIDGET(dialog));
 }
 
 /************************************************************************//**
@@ -369,15 +345,16 @@ static void option_color_select_callback(GtkButton *button, gpointer data)
 
   chooser = gtk_color_chooser_widget_new();
   g_object_set_data(G_OBJECT(dialog), "chooser", chooser);
-  gtk_box_insert_child_after(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
-                             chooser, NULL);
-  if (current_color != NULL) {
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), chooser,
+                     FALSE, FALSE, 0);
+  if (current_color) {
     gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(chooser), current_color);
   }
 
-  gtk_widget_set_visible(dialog, TRUE);
+  gtk_widget_show_all(dialog);
 }
 
+
 /************************************************************************//**
   Creates a new option dialog.
 ****************************************************************************/
@@ -410,6 +387,7 @@ option_dialog_new(const char *name, const struct option_set *poptset)
 
   /* Shell */
   setup_dialog(pdialog->shell, toplevel);
+  gtk_window_set_position(GTK_WINDOW(pdialog->shell), GTK_WIN_POS_MOUSE);
   gtk_window_set_default_size(GTK_WINDOW(pdialog->shell),
                               opt_dlog_width,
                               opt_dlog_height);
@@ -418,8 +396,8 @@ option_dialog_new(const char *name, const struct option_set *poptset)
   g_signal_connect(pdialog->shell, "destroy",
                    G_CALLBACK(option_dialog_destroy_callback), pdialog);
 
-  gtk_box_insert_child_after(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pdialog->shell))),
-                             pdialog->notebook, NULL);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pdialog->shell))),
+                     pdialog->notebook, TRUE, TRUE, 0);
 
   /* Add the options. */
   options_iterate(poptset, poption) {
@@ -429,7 +407,7 @@ option_dialog_new(const char *name, const struct option_set *poptset)
   option_dialog_reorder_notebook(pdialog);
 
   /* Show the widgets. */
-  gtk_widget_set_visible(pdialog->shell, TRUE);
+  gtk_widget_show_all(pdialog->shell);
 
   return pdialog;
 }
@@ -452,7 +430,7 @@ static void option_dialog_destroy(struct option_dialog *pdialog)
   if (NULL != shell) {
     /* Maybe already destroyed, see also option_dialog_destroy_callback(). */
     pdialog->shell = NULL;
-    gtk_window_destroy(GTK_WINDOW(shell));
+    gtk_widget_destroy(shell);
   }
 
   free(pdialog->vboxes);
@@ -516,10 +494,7 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
                                      bool reorder_notebook)
 {
   const int category = option_category(poption);
-  GtkWidget *main_hbox, *label, *w = NULL;
-  int main_col = 0;
-  GtkGesture *gesture;
-  GtkEventController *controller;
+  GtkWidget *main_hbox, *label, *ebox, *w = NULL;
 
   fc_assert(NULL == option_get_gui_data(poption));
 
@@ -527,11 +502,10 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
   if (NULL == pdialog->vboxes[category]) {
     GtkWidget *sw;
 
-    sw = gtk_scrolled_window_new();
+    sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                    GTK_POLICY_NEVER,
                                    GTK_POLICY_AUTOMATIC);
-    gtk_scrolled_window_set_propagate_natural_height(GTK_SCROLLED_WINDOW(sw), TRUE);
     g_object_set_data(G_OBJECT(sw), "category", GINT_TO_POINTER(category));
     gtk_notebook_append_page(GTK_NOTEBOOK(pdialog->notebook), sw,
                              gtk_label_new_with_mnemonic
@@ -541,34 +515,28 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
       option_dialog_reorder_notebook(pdialog);
     }
 
-    pdialog->vboxes[category] = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
-    gtk_widget_set_margin_bottom(pdialog->vboxes[category], 8);
-    gtk_widget_set_margin_end(pdialog->vboxes[category], 8);
-    gtk_widget_set_margin_start(pdialog->vboxes[category], 8);
-    gtk_widget_set_margin_top(pdialog->vboxes[category], 8);
+    pdialog->vboxes[category] = gtk_grid_new();
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(pdialog->vboxes[category]),
+                                   GTK_ORIENTATION_VERTICAL);
+    g_object_set(pdialog->vboxes[category], "margin", 8, NULL);
     gtk_widget_set_hexpand(pdialog->vboxes[category], TRUE);
-    gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(sw),
-                                  pdialog->vboxes[category]);
+    gtk_container_add(GTK_CONTAINER(sw), pdialog->vboxes[category]);
 
-    gtk_widget_set_visible(sw, TRUE);
+    gtk_widget_show_all(sw);
   }
   pdialog->box_children[category]++;
 
+  ebox = gtk_event_box_new();
+  gtk_widget_set_tooltip_text(ebox, option_help_text(poption));
+  gtk_container_add(GTK_CONTAINER(pdialog->vboxes[category]), ebox);
+  g_signal_connect(ebox, "button_press_event",
+                   G_CALLBACK(option_button_press_callback), poption);
+
   main_hbox = gtk_grid_new();
   label = gtk_label_new(option_description(poption));
-  gtk_widget_set_margin_bottom(label, 2);
-  gtk_widget_set_margin_end(label, 2);
-  gtk_widget_set_margin_start(label, 2);
-  gtk_widget_set_margin_top(label, 2);
-  gtk_grid_attach(GTK_GRID(main_hbox), label, main_col++, 0, 1, 1);
-  gtk_widget_set_tooltip_text(main_hbox, option_help_text(poption));
-  gesture = gtk_gesture_click_new();
-  gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gesture), 3);
-  controller = GTK_EVENT_CONTROLLER(gesture);
-  g_signal_connect(controller, "pressed",
-                   G_CALLBACK(option_button_press_callback), poption);
-  gtk_widget_add_controller(main_hbox, controller);
-  gtk_box_append(GTK_BOX(pdialog->vboxes[category]), main_hbox);
+  g_object_set(label, "margin", 2, NULL);
+  gtk_container_add(GTK_CONTAINER(main_hbox), label);
+  gtk_container_add(GTK_CONTAINER(ebox), main_hbox);
 
   switch (option_type(poption)) {
   case OT_BOOLEAN:
@@ -633,7 +601,7 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
       grid = gtk_grid_new();
       gtk_grid_set_column_spacing(GTK_GRID(grid), 4);
       gtk_grid_set_row_homogeneous(GTK_GRID(grid), TRUE);
-      gtk_frame_set_child(GTK_FRAME(w), grid);
+      gtk_container_add(GTK_CONTAINER(w), grid);
       for (i = 0; i < strvec_size(values); i++) {
         check = gtk_check_button_new();
         gtk_grid_attach(GTK_GRID(grid), check, 0, i, 1, 1);
@@ -654,7 +622,6 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
   case OT_COLOR:
     {
       GtkWidget *button;
-      int grid_col = 0;
 
       w = gtk_grid_new();
       gtk_grid_set_column_spacing(GTK_GRID(w), 4);
@@ -662,7 +629,7 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
 
       /* Foreground color selector button. */
       button = gtk_button_new();
-      gtk_grid_attach(GTK_GRID(w), button, grid_col++, 0, 1, 1);
+      gtk_container_add(GTK_CONTAINER(w), button);
       gtk_widget_set_tooltip_text(GTK_WIDGET(button),
                                   _("Select the text color"));
       g_object_set_data(G_OBJECT(w), "fg_button", button);
@@ -671,7 +638,7 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
 
       /* Background color selector button. */
       button = gtk_button_new();
-      gtk_grid_attach(GTK_GRID(w), button, grid_col++, 0, 1, 1);
+      gtk_container_add(GTK_CONTAINER(w), button);
       gtk_widget_set_tooltip_text(GTK_WIDGET(button),
                                   _("Select the background color"));
       g_object_set_data(G_OBJECT(w), "bg_button", button);
@@ -692,14 +659,13 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
     log_error("Failed to create a widget for option %d \"%s\".",
               option_number(poption), option_name(poption));
   } else {
-    g_object_set_data(G_OBJECT(w), "main_widget", main_hbox);
-    g_object_set_data(G_OBJECT(w), "parent_of_main", pdialog->vboxes[category]);
+    g_object_set_data(G_OBJECT(w), "main_widget", ebox);
     gtk_widget_set_hexpand(w, TRUE);
     gtk_widget_set_halign(w, GTK_ALIGN_END);
-    gtk_grid_attach(GTK_GRID(main_hbox), w, main_col++, 0, 1, 1);
+    gtk_container_add(GTK_CONTAINER(main_hbox), w);
   }
 
-  gtk_widget_set_visible(main_hbox, TRUE);
+  gtk_widget_show_all(ebox);
 
   /* Set as current value. */
   option_dialog_option_refresh(poption);
@@ -717,8 +683,7 @@ static void option_dialog_option_remove(struct option_dialog *pdialog,
     const int category = option_category(poption);
 
     option_set_gui_data(poption, NULL);
-    gtk_box_remove(GTK_BOX(g_object_get_data(object, "parent_of_main")),
-                   GTK_WIDGET(g_object_get_data(object, "main_widget")));
+    gtk_widget_destroy(GTK_WIDGET(g_object_get_data(object, "main_widget")));
 
     /* Remove category if needed. */
     if (0 == --pdialog->box_children[category]) {
@@ -734,9 +699,9 @@ static void option_dialog_option_remove(struct option_dialog *pdialog,
 static inline void option_dialog_option_bool_set(struct option *poption,
                                                  bool value)
 {
-  gtk_check_button_set_active(GTK_CHECK_BUTTON
-                              (option_get_gui_data(poption)),
-                              value);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+                               (option_get_gui_data(poption)),
+                               value);
 }
 
 /************************************************************************//**
@@ -755,14 +720,11 @@ static inline void option_dialog_option_int_set(struct option *poption,
 static inline void option_dialog_option_str_set(struct option *poption,
                                                 const char *string)
 {
-  GtkWidget *wdg = option_get_gui_data(poption);
-
   if (NULL != option_str_values(poption)) {
-    GtkWidget *child = gtk_combo_box_get_child(GTK_COMBO_BOX(wdg));
-
-    gtk_entry_buffer_set_text(gtk_entry_get_buffer(GTK_ENTRY(child)), string, -1);
+    gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN
+                       (option_get_gui_data(poption)))), string);
   } else {
-    gtk_entry_buffer_set_text(gtk_entry_get_buffer(GTK_ENTRY(wdg)), string, -1);
+    gtk_entry_set_text(GTK_ENTRY(option_get_gui_data(poption)), string);
   }
 }
 
@@ -802,8 +764,8 @@ static inline void option_dialog_option_bitwise_set(struct option *poption,
   int bit;
 
   for (bit = 0; NULL != iter; iter = g_list_next(iter), bit++) {
-    gtk_check_button_set_active(GTK_CHECK_BUTTON(iter->data),
-                                value & (1 << bit));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(iter->data),
+                                 value & (1 << bit));
   }
 }
 
@@ -933,8 +895,8 @@ static void option_dialog_option_apply(struct option *poption)
 
   switch (option_type(poption)) {
   case OT_BOOLEAN:
-    (void) option_bool_set(poption, gtk_check_button_get_active
-                           (GTK_CHECK_BUTTON(w)));
+    (void) option_bool_set(poption, gtk_toggle_button_get_active
+                           (GTK_TOGGLE_BUTTON(w)));
     break;
 
   case OT_INTEGER:
@@ -944,11 +906,10 @@ static void option_dialog_option_apply(struct option *poption)
 
   case OT_STRING:
     if (NULL != option_str_values(poption)) {
-      (void) option_str_set(poption,
-                            gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(w)));
+      (void) option_str_set(poption, gtk_entry_get_text
+                            (GTK_ENTRY(gtk_bin_get_child(GTK_BIN(w)))));
     } else {
-      (void) option_str_set(poption, gtk_entry_buffer_get_text(
-                                                 gtk_entry_get_buffer(GTK_ENTRY(w))));
+      (void) option_str_set(poption, gtk_entry_get_text(GTK_ENTRY(w)));
     }
     break;
 
@@ -974,7 +935,7 @@ static void option_dialog_option_apply(struct option *poption)
       int bit;
 
       for (bit = 0; NULL != iter; iter = g_list_next(iter), bit++) {
-        if (gtk_check_button_get_active(GTK_CHECK_BUTTON(iter->data))) {
+        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(iter->data))) {
           value |= 1 << bit;
         }
       }
