@@ -75,7 +75,7 @@ bool inputline_has_focus(void)
 }
 
 /**********************************************************************//**
-  Gives the focus to the intput line.
+  Gives the focus to the input line.
 **************************************************************************/
 void inputline_grab_focus(void)
 {
@@ -107,7 +107,7 @@ static gboolean il_lost_focus(GtkEventControllerFocus *controller,
 static gboolean il_gained_focus(GtkEventControllerFocus *controller,
                                 gpointer data)
 {
-  menus_disable_unit_commands();
+  menus_disable_char_commands();
 
   return TRUE;
 }
@@ -188,7 +188,7 @@ static void inputline_return(GtkEntry *w, gpointer data)
     }
 
     genlist_prepend(history_list, fc_strdup(theinput));
-    history_pos=-1;
+    history_pos = -1;
   }
 
   gtk_entry_buffer_set_text(buffer, "", -1);
@@ -1115,6 +1115,7 @@ static void color_set(GObject *object, const gchar *color_target,
         cairo_surface_t *surface = cairo_image_surface_create(
             CAIRO_FORMAT_RGB24, 16, 16);
         cairo_t *cr = cairo_create(surface);
+
         gdk_cairo_set_source_rgba(cr, current_color);
         cairo_paint(cr);
         cairo_destroy(cr);
@@ -1123,7 +1124,7 @@ static void color_set(GObject *object, const gchar *color_target,
       }
       image = gtk_image_new_from_pixbuf(pixbuf);
       gtk_button_set_child(button, image);
-      gtk_widget_show(image);
+      gtk_widget_set_visible(image, TRUE);
       g_object_unref(G_OBJECT(pixbuf));
     }
   }
@@ -1186,7 +1187,7 @@ static void select_color_callback(GtkButton *button, gpointer data)
     gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(chooser), current_color);
   }
 
-  gtk_widget_show(dialog);
+  gtk_widget_set_visible(dialog, TRUE);
   g_free(buf);
 }
 
@@ -1209,7 +1210,7 @@ static gboolean move_toolkit(GtkWidget *toolkit_view, gpointer data)
     /* N.B.: We need to hide/show the toolbar to reset the sensitivity
      * of the tool buttons. */
     if (ptoolkit->toolbar_displayed) {
-      gtk_widget_hide(ptoolkit->toolbar);
+      gtk_widget_set_visible(ptoolkit->toolbar, FALSE);
     }
     g_object_ref(ptoolkit->main_widget); /* Make sure reference count stays above 0
                                           * during the transition to new parent. */
@@ -1217,16 +1218,16 @@ static gboolean move_toolkit(GtkWidget *toolkit_view, gpointer data)
     gtk_box_append(GTK_BOX(toolkit_view), ptoolkit->main_widget);
     g_object_unref(ptoolkit->main_widget);
     if (ptoolkit->toolbar_displayed) {
-      gtk_widget_show(ptoolkit->toolbar);
+      gtk_widget_set_visible(ptoolkit->toolbar, TRUE);
     }
 
     if (!gtk_widget_get_parent(button_box)) {
       /* Attach to the toolkit button_box. */
       gtk_box_append(GTK_BOX(ptoolkit->button_box), button_box);
     }
-    gtk_widget_show(button_box);
+    gtk_widget_set_visible(button_box, TRUE);
     if (!ptoolkit->toolbar_displayed) {
-      gtk_widget_hide(ptoolkit->toolbar);
+      gtk_widget_set_visible(ptoolkit->toolbar, FALSE);
     }
 
     /* Hide all other buttons boxes. */
@@ -1234,7 +1235,7 @@ static gboolean move_toolkit(GtkWidget *toolkit_view, gpointer data)
          iter != NULL;
          iter = gtk_widget_get_next_sibling(iter)) {
       if (iter != button_box) {
-        gtk_widget_hide(iter);
+        gtk_widget_set_visible(iter, FALSE);
       }
     }
 
@@ -1242,7 +1243,7 @@ static gboolean move_toolkit(GtkWidget *toolkit_view, gpointer data)
     /* First time attached to a parent. */
     gtk_box_append(GTK_BOX(toolkit_view), ptoolkit->main_widget);
     gtk_box_append(GTK_BOX(ptoolkit->button_box), button_box);
-    gtk_widget_show(ptoolkit->main_widget);
+    gtk_widget_set_visible(ptoolkit->main_widget, TRUE);
   }
 
   return FALSE;
@@ -1262,7 +1263,7 @@ static gboolean set_toolbar_visibility(GtkWidget *w, gpointer data)
       gtk_toggle_button_set_active(button, TRUE);
     } else {
       /* Ensure the widget is visible. */
-      gtk_widget_show(ptoolkit->toolbar);
+      gtk_widget_set_visible(ptoolkit->toolbar, TRUE);
     }
   } else {
     if (gtk_toggle_button_get_active(button)) {
@@ -1270,7 +1271,7 @@ static gboolean set_toolbar_visibility(GtkWidget *w, gpointer data)
       gtk_toggle_button_set_active(button, FALSE);
     } else {
       /* Ensure the widget is not visible. */
-      gtk_widget_hide(ptoolkit->toolbar);
+      gtk_widget_set_visible(ptoolkit->toolbar, FALSE);
     }
   }
 
@@ -1285,14 +1286,14 @@ static void button_toggled(GtkToggleButton *button, gpointer data)
   struct inputline_toolkit *ptoolkit = (struct inputline_toolkit *) data;
 
   if (gtk_toggle_button_get_active(button)) {
-    gtk_widget_show(ptoolkit->toolbar);
+    gtk_widget_set_visible(ptoolkit->toolbar, TRUE);
     ptoolkit->toolbar_displayed = TRUE;
     if (chatline_is_scrolled_to_bottom()) {
       /* Make sure to be still at the end. */
       chatline_scroll_to_bottom(TRUE);
     }
   } else {
-    gtk_widget_hide(ptoolkit->toolbar);
+    gtk_widget_set_visible(ptoolkit->toolbar, FALSE);
     ptoolkit->toolbar_displayed = FALSE;
   }
 }
