@@ -37,7 +37,7 @@
 #include "client_main.h"
 #include "options.h"
 
-/* client/gui-gtk-3.22 */
+/* client/gui-gtk-4.0 */
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
@@ -267,15 +267,18 @@ static struct intel_dialog *create_intel_dialog(struct player *p)
   g_signal_connect(shell, "destroy",
                    G_CALLBACK(intel_destroy_callback), pdialog);
   g_signal_connect(shell, "response",
-                   G_CALLBACK(gtk_widget_destroy), NULL);
+                   G_CALLBACK(gtk_window_destroy), NULL);
 
   notebook = gtk_notebook_new();
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
-  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(shell))), notebook);
- 
+  gtk_box_append(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(shell))), notebook);
+
   /* overview tab. */
   table = gtk_grid_new();
-  g_object_set(table, "margin", 6, NULL);
+  gtk_widget_set_margin_bottom(table, 6);
+  gtk_widget_set_margin_end(table, 6);
+  gtk_widget_set_margin_start(table, 6);
+  gtk_widget_set_margin_top(table, 6);
 
   gtk_grid_set_row_spacing(GTK_GRID(table), 2);
   gtk_grid_set_column_spacing(GTK_GRID(table), 12);
@@ -305,11 +308,17 @@ static struct intel_dialog *create_intel_dialog(struct player *p)
   pdialog->diplstates = gtk_tree_store_new(1, G_TYPE_STRING);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pdialog->diplstates));
-  g_object_set(view, "margin", 6, NULL);
+  gtk_widget_set_margin_bottom(view, 6);
+  gtk_widget_set_margin_end(view, 6);
+  gtk_widget_set_margin_start(view, 6);
+  gtk_widget_set_margin_top(view, 6);
   gtk_widget_set_hexpand(view, TRUE);
   gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(pdialog->diplstates);
-  gtk_container_set_border_width(GTK_CONTAINER(view), 6);
+  gtk_widget_set_margin_start(view, 6);
+  gtk_widget_set_margin_end(view, 6);
+  gtk_widget_set_margin_top(view, 6);
+  gtk_widget_set_margin_bottom(view, 6);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
   rend = gtk_cell_renderer_text_new();
@@ -319,10 +328,9 @@ static struct intel_dialog *create_intel_dialog(struct player *p)
 
   gtk_tree_view_expand_all(GTK_TREE_VIEW(view));
 
-  sw = gtk_scrolled_window_new(NULL,NULL);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-				      GTK_SHADOW_ETCHED_IN);
-  gtk_container_add(GTK_CONTAINER(sw), view);
+  sw = gtk_scrolled_window_new();
+  gtk_scrolled_window_set_has_frame(GTK_SCROLLED_WINDOW(sw), TRUE);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(sw), view);
 
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -336,11 +344,17 @@ static struct intel_dialog *create_intel_dialog(struct player *p)
       1, GTK_SORT_ASCENDING);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pdialog->techs));
-  g_object_set(view, "margin", 6, NULL);
+  gtk_widget_set_margin_bottom(view, 6);
+  gtk_widget_set_margin_end(view, 6);
+  gtk_widget_set_margin_start(view, 6);
+  gtk_widget_set_margin_top(view, 6);
   gtk_widget_set_hexpand(view, TRUE);
   gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(pdialog->techs);
-  gtk_container_set_border_width(GTK_CONTAINER(view), 6);
+  gtk_widget_set_margin_start(view, 6);
+  gtk_widget_set_margin_end(view, 6);
+  gtk_widget_set_margin_top(view, 6);
+  gtk_widget_set_margin_bottom(view, 6);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
   rend = gtk_cell_renderer_toggle_new();
@@ -353,10 +367,9 @@ static struct intel_dialog *create_intel_dialog(struct player *p)
     "text", 1, NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
 
-  sw = gtk_scrolled_window_new(NULL,NULL);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-				      GTK_SHADOW_ETCHED_IN);
-  gtk_container_add(GTK_CONTAINER(sw), view);
+  sw = gtk_scrolled_window_new();
+  gtk_scrolled_window_set_has_frame(GTK_SCROLLED_WINDOW(sw), TRUE);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(sw), view);
 
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -364,7 +377,8 @@ static struct intel_dialog *create_intel_dialog(struct player *p)
   label = gtk_label_new_with_mnemonic(_("_Techs"));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw, label);
 
-  gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(shell)));
+  gtk_widget_set_visible(gtk_dialog_get_content_area(GTK_DIALOG(shell)),
+                         TRUE);
 
   dialog_list_prepend(dialog_list, pdialog);
 
@@ -399,7 +413,7 @@ static struct intel_wonder_dialog *create_intel_wonder_dialog(struct player *p)
   g_signal_connect(shell, "destroy",
                    G_CALLBACK(intel_wonder_destroy_callback), pdialog);
   g_signal_connect(shell, "response",
-                   G_CALLBACK(gtk_widget_destroy), NULL);
+                   G_CALLBACK(gtk_window_destroy), NULL);
 
 
   pdialog->rule = gtk_label_new("-");
@@ -414,11 +428,17 @@ static struct intel_wonder_dialog *create_intel_wonder_dialog(struct player *p)
   gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(pdialog->wonders),
                                        3, GTK_SORT_DESCENDING);
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pdialog->wonders));
-  g_object_set(view, "margin", 6, NULL);
+  gtk_widget_set_margin_bottom(view, 6);
+  gtk_widget_set_margin_end(view, 6);
+  gtk_widget_set_margin_start(view, 6);
+  gtk_widget_set_margin_top(view, 6);
   gtk_widget_set_hexpand(view, TRUE);
   gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(pdialog->wonders);
-  gtk_container_set_border_width(GTK_CONTAINER(view), 6);
+  gtk_widget_set_margin_start(view, 6);
+  gtk_widget_set_margin_end(view, 6);
+  gtk_widget_set_margin_top(view, 6);
+  gtk_widget_set_margin_bottom(view, 6);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
   rend = gtk_cell_renderer_text_new();
@@ -433,22 +453,21 @@ static struct intel_wonder_dialog *create_intel_wonder_dialog(struct player *p)
                                               rend, "text", 1,
                                               NULL);
 
-  sw = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-                                      GTK_SHADOW_ETCHED_IN);
+  sw = gtk_scrolled_window_new();
+  gtk_scrolled_window_set_has_frame(GTK_SCROLLED_WINDOW(sw), TRUE);
   box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
-  gtk_container_add(GTK_CONTAINER(box), pdialog->rule);
-  gtk_container_add(GTK_CONTAINER(box), view);
-  gtk_container_add(GTK_CONTAINER(sw), box);
+  gtk_box_append(GTK_BOX(box), pdialog->rule);
+  gtk_box_append(GTK_BOX(box), view);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(sw), box);
 
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
 
-  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(shell))),
-                    sw);
+  gtk_box_append(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(shell))), sw);
 
-  gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(shell)));
+  gtk_widget_set_visible(gtk_dialog_get_content_area(GTK_DIALOG(shell)),
+                         TRUE);
 
   wonder_dialog_list_prepend(wonder_dialogs, pdialog);
 
@@ -669,6 +688,7 @@ void update_intel_wonder_dialog(struct player *p)
 
         if (wonder_is_built(p, impr)) {
           struct city *pcity = city_from_wonder(p, impr);
+
           if (pcity) {
             cityname = city_name_get(pcity);
           } else {
