@@ -18,7 +18,10 @@
 /* client */
 #include "menu_g.h"
 
-GtkWidget *setup_menus(GtkWidget *window);
-void menus_free(void);
+void enable_menus(bool enable);
 
-#endif  /* FC__MENU_H */
+void menus_set_initial_toggle_values(void);
+void menus_disable_unit_commands(void);
+void menus_disable_char_commands(void);
+
+#endif /* FC__MENU_H */
