@@ -27,19 +27,18 @@
 /* client */
 #include "tilespec.h"
 
-/* client/gui-gtk-3.22 */
+/* client/gui-gtk-4.0 */
 #include "gui_main.h"
 #include "gui_stuff.h"
 
 #include "dialogs_g.h"
 
 extern char forced_tileset_name[512];
-static void tileset_suggestion_callback(GtkWidget *dlg, gint arg);
 
 /************************************************************************//**
   Callback either loading suggested tileset or doing nothing
 ****************************************************************************/
-static void tileset_suggestion_callback(GtkWidget *dlg, gint arg)
+static void tileset_suggestion_response(gint arg)
 {
   if (arg == GTK_RESPONSE_YES) {
     /* User accepted tileset loading */
@@ -79,17 +78,15 @@ void popup_tileset_suggestion_dialog(void)
               game.control.preferred_tileset, tileset_basename(tileset));
 
   label = gtk_label_new(buf);
-  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), label);
+  gtk_box_append(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+                 label);
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
-  gtk_widget_show(label);
-
-  g_signal_connect(dialog, "response",
-                   G_CALLBACK(tileset_suggestion_callback), NULL);
+  gtk_widget_set_visible(label, TRUE);
 
   /* In case incoming rulesets are incompatible with current tileset
    * we need to block their receive before user has accepted loading
    * of the correct tileset. */
-  gtk_dialog_run(GTK_DIALOG(dialog));
+  tileset_suggestion_response(blocking_dialog(dialog));
 
-  gtk_widget_destroy(dialog);
+  gtk_window_destroy(GTK_WINDOW(dialog));
 }
