@@ -26,7 +26,7 @@
 #include "options.h"
 #include "voteinfo.h"
 
-/* client/gui-gtk-3.22 */
+/* client/gui-gtk-4.0 */
 #include "chatline.h"
 #include "pages.h"
 
@@ -56,7 +56,7 @@ static void voteinfo_bar_do_vote_callback(GtkWidget *w, gpointer userdata)
 {
   enum client_vote_type vote;
   struct voteinfo *vi;
-  
+
   vote = GPOINTER_TO_INT(userdata);
   vi = voteinfo_queue_get_current(NULL);
 
@@ -91,31 +91,33 @@ static void voteinfo_bar_destroy(GtkWidget *w, gpointer userdata)
 **************************************************************************/
 GtkWidget *voteinfo_bar_new(bool split_bar)
 {
-  GtkWidget *label, *button, *vbox, *hbox, *arrow;
+  GtkWidget *label, *button, *vgrid, *hgrid;
   struct voteinfo_bar *vib;
   const int BUTTON_HEIGHT = 12;
+  int grid_row = 0;
+  int grid_col = 0;
 
   vib = fc_calloc(1, sizeof(struct voteinfo_bar));
 
   if (!split_bar) {
-    hbox = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(hbox), 4);
-    g_object_set_data(G_OBJECT(hbox), "voteinfo_bar", vib);
-    g_signal_connect(hbox, "destroy", G_CALLBACK(voteinfo_bar_destroy), vib);
-    vib->box = hbox;
-    vbox = NULL;        /* The compiler may require it. */
+    hgrid = gtk_grid_new();
+    gtk_grid_set_column_spacing(GTK_GRID(hgrid), 4);
+    g_object_set_data(G_OBJECT(hgrid), "voteinfo_bar", vib);
+    g_signal_connect(hgrid, "destroy", G_CALLBACK(voteinfo_bar_destroy), vib);
+    vib->box = hgrid;
+    vgrid = NULL;        /* The compiler may require it. */
   } else {
-    vbox = gtk_grid_new();
-    gtk_grid_set_row_homogeneous(GTK_GRID(vbox), TRUE);
-    gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
+    vgrid = gtk_grid_new();
+    gtk_grid_set_row_homogeneous(GTK_GRID(vgrid), TRUE);
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(vgrid),
                                    GTK_ORIENTATION_VERTICAL);
-    gtk_grid_set_row_spacing(GTK_GRID(vbox), 4);
-    g_object_set_data(G_OBJECT(vbox), "voteinfo_bar", vib);
-    g_signal_connect(vbox, "destroy", G_CALLBACK(voteinfo_bar_destroy), vib);
-    vib->box = vbox;
-    hbox = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(hbox), 4);
-    gtk_container_add(GTK_CONTAINER(vbox), hbox);
+    gtk_grid_set_row_spacing(GTK_GRID(vgrid), 4);
+    g_object_set_data(G_OBJECT(vgrid), "voteinfo_bar", vib);
+    g_signal_connect(vgrid, "destroy", G_CALLBACK(voteinfo_bar_destroy), vib);
+    vib->box = vgrid;
+    hgrid = gtk_grid_new();
+    gtk_grid_set_column_spacing(GTK_GRID(hgrid), 4);
+    gtk_grid_attach(GTK_GRID(vgrid), hgrid, 0, grid_row++, 1, 1);
   }
 
   label = gtk_label_new("");
@@ -127,30 +129,26 @@ GtkWidget *voteinfo_bar_new(bool split_bar)
   gtk_widget_set_margin_top(label, 4);
   gtk_widget_set_margin_bottom(label, 4);
   gtk_label_set_max_width_chars(GTK_LABEL(label), 80);
-  gtk_container_add(GTK_CONTAINER(hbox), label);
+  gtk_grid_attach(GTK_GRID(hgrid), label, grid_col++, 0, 1, 1);
   gtk_widget_set_name(label, "vote label");
   vib->label = label;
 
-  arrow = gtk_image_new_from_icon_name("media-seek-backward",
-                                       GTK_ICON_SIZE_SMALL_TOOLBAR);
-  gtk_widget_set_halign(arrow, GTK_ALIGN_CENTER);
-  gtk_widget_set_valign(arrow, GTK_ALIGN_START);
-
   if (split_bar) {
-    hbox = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(hbox), 4);
-    gtk_container_add(GTK_CONTAINER(vbox), hbox);
+    hgrid = gtk_grid_new();
+    grid_col = 0;
+    gtk_grid_set_column_spacing(GTK_GRID(hgrid), 4);
+    gtk_grid_attach(GTK_GRID(vgrid), hgrid, 0, grid_row++, 1, 1);
   }
 
   button = gtk_button_new();
   gtk_widget_set_margin_end(button, 16);
   g_signal_connect(button, "clicked",
                    G_CALLBACK(voteinfo_bar_next_callback), NULL);
-  gtk_button_set_image(GTK_BUTTON(button), arrow);
+  gtk_button_set_icon_name(GTK_BUTTON(button), "media-seek-backward");
   gtk_widget_set_size_request(button, -1, BUTTON_HEIGHT);
-  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+  gtk_button_set_has_frame(GTK_BUTTON(button), FALSE);
   gtk_widget_set_focus_on_click(button, FALSE);
-  gtk_container_add(GTK_CONTAINER(hbox), button);
+  gtk_grid_attach(GTK_GRID(hgrid), button, grid_col++, 0, 1, 1);
   vib->next_button = button;
 
   button = gtk_button_new_with_mnemonic(_("_YES"));
@@ -158,14 +156,14 @@ GtkWidget *voteinfo_bar_new(bool split_bar)
                    G_CALLBACK(voteinfo_bar_do_vote_callback),
                    GINT_TO_POINTER(CVT_YES));
   gtk_widget_set_focus_on_click(button, FALSE);
-  gtk_container_add(GTK_CONTAINER(hbox), button);
+  gtk_grid_attach(GTK_GRID(hgrid), button, grid_col++, 0, 1, 1);
   gtk_widget_set_name(button, "vote yes button");
   vib->yes_button = button;
 
   label = gtk_label_new("0");
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_container_add(GTK_CONTAINER(hbox), label);
+  gtk_grid_attach(GTK_GRID(hgrid), label, grid_col++, 0, 1, 1);
   vib->yes_count_label = label;
 
   button = gtk_button_new_with_mnemonic(_("_NO"));
@@ -173,14 +171,14 @@ GtkWidget *voteinfo_bar_new(bool split_bar)
                    G_CALLBACK(voteinfo_bar_do_vote_callback),
                    GINT_TO_POINTER(CVT_NO));
   gtk_widget_set_focus_on_click(button, FALSE);
-  gtk_container_add(GTK_CONTAINER(hbox), button);
+  gtk_grid_attach(GTK_GRID(hgrid), button, grid_col++, 0, 1, 1);
   gtk_widget_set_name(button, "vote no button");
   vib->no_button = button;
 
   label = gtk_label_new("0");
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_container_add(GTK_CONTAINER(hbox), label);
+  gtk_grid_attach(GTK_GRID(hgrid), label, grid_col++, 0, 1, 1);
   vib->no_count_label = label;
 
   button = gtk_button_new_with_mnemonic(_("_ABSTAIN"));
@@ -188,20 +186,20 @@ GtkWidget *voteinfo_bar_new(bool split_bar)
                    G_CALLBACK(voteinfo_bar_do_vote_callback),
                    GINT_TO_POINTER(CVT_ABSTAIN));
   gtk_widget_set_focus_on_click(button, FALSE);
-  gtk_container_add(GTK_CONTAINER(hbox), button);
+  gtk_grid_attach(GTK_GRID(hgrid), button, grid_col++, 0, 1, 1);
   gtk_widget_set_name(button, "vote abstain button");
   vib->abstain_button = button;
 
   label = gtk_label_new("0");
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_container_add(GTK_CONTAINER(hbox), label);
+  gtk_grid_attach(GTK_GRID(hgrid), label, grid_col++, 0, 1, 1);
   vib->abstain_count_label = label;
 
   label = gtk_label_new("/0");
   gtk_widget_set_halign(label, GTK_ALIGN_START);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_container_add(GTK_CONTAINER(hbox), label);
+  gtk_grid_attach(GTK_GRID(hgrid), label, grid_col++, 0, 1, 1);
   vib->voter_count_label = label;
 
   return vib->box;
@@ -231,7 +229,7 @@ void voteinfo_gui_update(void)
   }
 
   if (!voteinfo_bar_can_be_shown()) {
-    gtk_widget_hide(vib->box);
+    gtk_widget_set_visible(vib->box, FALSE);
     return;
   }
 
@@ -308,10 +306,10 @@ void voteinfo_gui_update(void)
   need_scroll = !gtk_widget_get_visible(vib->box)
     && chatline_is_scrolled_to_bottom();
 
-  gtk_widget_show_all(vib->box);
+  gtk_widget_set_visible(vib->box, TRUE);
 
   if (vote_count <= 1) {
-    gtk_widget_hide(vib->next_button);
+    gtk_widget_set_visible(vib->next_button, FALSE);
   }
 
   if (need_scroll) {
