@@ -71,8 +71,8 @@ extern int last_center_enemy_city;
 bool is_point_in_area(int x, int y, int px, int py, int pxe, int pye)
 {
   if (x >= px && y >= py && x <= pxe && y <= pye) {
-      return true;
-    }
+    return true;
+  }
   return false;
 }
 
@@ -94,7 +94,7 @@ void draw_calculated_trade_routes(QPainter *painter)
     return;
   }
   pcolor = get_color(tileset, COLOR_MAPVIEW_TRADE_ROUTES_NO_BUILT);
-  /* Draw calculated trade routes */
+  // Draw calculated trade routes
   if (gui_options.draw_city_trade_routes) {
 
     foreach (qgilles, gui()->trade_gen.lines) {
@@ -105,7 +105,7 @@ void draw_calculated_trade_routes(QPainter *painter)
       tile_to_canvas_pos(&x1, &y1, map_zoom, qgilles.t1);
       tile_to_canvas_pos(&x2, &y2, map_zoom, qgilles.t2);
 
-      /* Dont draw if route was already established */
+      // Dont draw if route was already established
       if (tile_city(qgilles.t1) && tile_city(qgilles.t2)
           && have_cities_trade_route(tile_city(qgilles.t1),
                                      tile_city(qgilles.t2))) {
@@ -134,7 +134,7 @@ void draw_calculated_trade_routes(QPainter *painter)
                         y2 + tileset_tile_height(tileset) / 2 - h);
     }
   }
-  /* Draw virtual cities */
+  // Draw virtual cities
   foreach (pcity, gui()->trade_gen.virtual_cities) {
     float canvas_x, canvas_y;
     if (pcity->tile != nullptr
@@ -334,9 +334,7 @@ void map_view::find_place(int pos_x, int pos_y, int &w, int &h, int wdth,
   bool cont_searching = false;
 
   if (recursive_nr >= 1000) {
-    /**
-     * give up searching position
-     */
+    // Give up searching position
     return;
   }
   /**
@@ -618,14 +616,14 @@ static void gui_to_overview(int *ovr_x, int *ovr_y, int gui_x, int gui_y)
   *ovr_y = floor((ntl_y - (double)gui_options.overview.map_y0)
            * OVERVIEW_TILE_SIZE);
 
-  if (current_topo_has_flag(TF_WRAPX)) {
+  if (current_wrap_has_flag(WRAP_X)) {
     *ovr_x = FC_WRAP(*ovr_x, NATURAL_WIDTH * OVERVIEW_TILE_SIZE);
   } else {
     if (MAP_IS_ISOMETRIC) {
       *ovr_x -= OVERVIEW_TILE_SIZE;
     }
   }
-  if (current_topo_has_flag(TF_WRAPY)) {
+  if (current_wrap_has_flag(WRAP_Y)) {
     *ovr_y = FC_WRAP(*ovr_y, NATURAL_HEIGHT * OVERVIEW_TILE_SIZE);
   }
 }
@@ -1214,7 +1212,7 @@ void update_map_canvas_scrollbars(void)
 **************************************************************************/
 void update_map_canvas_scrollbars_size(void)
 {
-  /* PORTME */
+  // PORTME
 }
 
 /**********************************************************************//**
@@ -1235,7 +1233,7 @@ void pixmap_put_overlay_tile(int canvas_x, int  canvas_y,
     return;
   }
 
-  /* PORTME */
+  // PORTME
 }
 
 /**********************************************************************//**
@@ -1257,7 +1255,7 @@ void put_cross_overlay_tile(struct tile *ptile)
 **************************************************************************/
 void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
 {
-  /* PORTME */
+  // PORTME
 }
 
 /**********************************************************************//**
@@ -1271,7 +1269,7 @@ void tileset_changed(void)
 
   update_unit_info_label(get_units_in_focus());
   destroy_city_dialog();
-  /* Update science report if open */
+  // Update science report if open
   if (gui()->is_repo_dlg_open("SCI")) {
     i = gui()->gimme_index_of("SCI");
     fc_assert(i != -1);
@@ -1319,7 +1317,7 @@ void overview_size_changed(void)
 **************************************************************************/
 void update_overview_scroll_window_pos(int x, int y)
 {
-  /* TODO: PORTME. */
+  // TODO: PORTME.
 }
 
 /**********************************************************************//**
