@@ -148,8 +148,8 @@ fc_shortcut default_shortcuts[] = {
     | Qt::ShiftModifier, _("Lock/unlock interface") },
   {SC_AUTOMATE, Qt::Key_A, Qt::AllButtons, Qt::NoModifier,
     _("Auto worker") },
-  {SC_PARADROP, Qt::Key_P, Qt::AllButtons, Qt::NoModifier,
-    _("Paradrop/clean pollution") },
+  {SC_CLEAN, Qt::Key_P, Qt::AllButtons, Qt::NoModifier,
+    _("Clean") },
   {SC_POPUP_COMB_INF, Qt::Key_F1, Qt::AllButtons, Qt::ControlModifier,
     _("Popup combat information") },
   {SC_RELOAD_THEME, Qt::Key_F5, Qt::AllButtons, Qt::ControlModifier
@@ -173,10 +173,11 @@ fc_shortcut default_shortcuts[] = {
   {SC_GOJOINCITY, Qt::Key_J, Qt::AllButtons, Qt::ShiftModifier,
    _("Go And Join City") },
   {SC_STACK_SIZE, Qt::Key_Plus, Qt::AllButtons, Qt::ControlModifier,
-   _("Unit Stack Size") }
+   _("Unit Stack Size") },
+  {SC_PARADROP, Qt::Key_J, Qt::AllButtons, Qt::NoModifier,
+   _("Paradrop") }
 };
 
-
 /**********************************************************************//**
   Returns shortcut as string (eg. for menu)
 **************************************************************************/
@@ -526,7 +527,8 @@ fc_sc_button::fc_sc_button(): QPushButton()
 **************************************************************************/
 fc_sc_button::fc_sc_button(fc_shortcut *s): QPushButton()
 {
-  sc_orig = s;
+  fc_shortcut *sc_orig = s;
+
   sc = new fc_shortcut;
   sc->id = sc_orig->id;
   sc->key = sc_orig->key;
@@ -554,14 +556,14 @@ void fc_sc_button::popup_error()
   QList<fc_shortcut_popup *> fsb_list;
   QString title;
 
-  /* wait until shortcut popup is destroyed */
+  // Wait until shortcut popup is destroyed
   fsb_list = findChildren<fc_shortcut_popup *>();
   if (fsb_list.count() > 0) {
     QTimer::singleShot(20, this, SLOT(popup_error()));
     return;
   }
 
-  /* TRANS: Given shortcut(%1) is already assigned */
+  // TRANS: Given shortcut(%1) is already assigned
   title = QString(_("%1 is already assigned to"))
                   .arg(shortcut_to_string(sc));
   scinfo = new hud_message_box(gui()->central_wdg);
