@@ -65,8 +65,7 @@ extern struct widget *options_button;
 static struct widget *begin_order_widget_list;
 static struct widget *end_order_widget_list;
 
-static struct widget *order_fallout_button;
-static struct widget *order_pollution_button;
+static struct widget *order_clean_button;
 static struct widget *order_airbase_button;
 static struct widget *order_fortress_button;
 static struct widget *order_build_add_to_city_button;
@@ -141,15 +140,12 @@ static int unit_order_callback(struct widget *order_widget)
     case ID_UNIT_ORDER_AIRBASE:
       key_unit_airbase();
       break;
-    case ID_UNIT_ORDER_POLLUTION:
-      key_unit_pollution();
+    case ID_UNIT_ORDER_CLEAN:
+      key_unit_clean();
       break;
     case ID_UNIT_ORDER_PARADROP:
       key_unit_paradrop();
       break;
-    case ID_UNIT_ORDER_FALLOUT:
-      key_unit_fallout();
-      break;
     case ID_UNIT_ORDER_SENTRY:
       key_unit_sentry();
       break;
@@ -730,20 +726,6 @@ void create_units_order_widgets(void)
   add_to_gui_list(ID_UNIT_ORDER_SENTRY, buf);
   /* --------- */
 
-  /* Clean Nuclear Fallout */
-  /* Label will be replaced by real_menus_update() before it's seen */
-  fc_snprintf(cbuf, sizeof(cbuf), "placeholder");
-  buf = create_themeicon(current_theme->o_fallout_icon, main_data.gui,
-                         WF_HIDDEN | WF_RESTORE_BACKGROUND
-                         | WF_WIDGET_HAS_INFO_LABEL);
-  set_wstate(buf, FC_WS_NORMAL);
-  buf->action = unit_order_callback;
-  buf->info_label = create_utf8_from_char_fonto(cbuf, FONTO_DEFAULT);
-  buf->key = SDLK_n;
-  order_fallout_button = buf;
-  add_to_gui_list(ID_UNIT_ORDER_FALLOUT, buf);
-  /* --------- */
-
   /* Paradrop */
   fc_snprintf(cbuf, sizeof(cbuf), "%s (%s)",
               action_id_name_translation(ACTION_PARADROP), "J");
@@ -757,18 +739,18 @@ void create_units_order_widgets(void)
   add_to_gui_list(ID_UNIT_ORDER_PARADROP, buf);
   /* --------- */
 
-  /* Clean Pollution */
+  /* Generic Clean */
   /* Label will be replaced by real_menus_update() before it's seen */
   fc_snprintf(cbuf, sizeof(cbuf), "placeholder");
-  buf = create_themeicon(current_theme->o_pollution_icon, main_data.gui,
+  buf = create_themeicon(current_theme->o_clean_icon, main_data.gui,
                          WF_HIDDEN | WF_RESTORE_BACKGROUND
                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(buf, FC_WS_NORMAL);
   buf->action = unit_order_callback;
   buf->info_label = create_utf8_from_char_fonto(cbuf, FONTO_DEFAULT);
   buf->key = SDLK_p;
-  order_pollution_button = buf;
-  add_to_gui_list(ID_UNIT_ORDER_POLLUTION, buf);
+  order_clean_button = buf;
+  add_to_gui_list(ID_UNIT_ORDER_CLEAN, buf);
   /* --------- */
 
   /* Build Airbase */
@@ -1153,6 +1135,7 @@ void real_menus_update(void)
           && (homecity = game_city_by_number(punit->homecity))
           && can_cities_trade(homecity, pcity)) {
         int revenue = get_caravan_enter_city_trade_bonus(homecity, pcity,
+                                                         unit_type_get(punit),
                                                          punit->carrying,
                                                          TRUE);
 
@@ -1287,42 +1270,27 @@ void real_menus_update(void)
         set_wflag(order_airbase_button, WF_HIDDEN);
       }
 
-      pextra = prev_extra_in_tile(ptile, ERM_CLEANPOLLUTION,
-                                  unit_owner(punit), punit);
-      if (pextra != NULL
-          && can_unit_do_activity_targeted_client(punit, ACTIVITY_POLLUTION, pextra)) {
-        time = turns_to_activity_done(ptile, ACTIVITY_POLLUTION, pextra,
-                                      punit);
-        /* TRANS: "Clean Pollution (P) 3 turns" */
-        fc_snprintf(cbuf, sizeof(cbuf), _("Clean %s (%s) %d %s"),
-                    extra_name_translation(pextra), "P", time,
-                    PL_("turn", "turns", time));
-        copy_chars_to_utf8_str(order_pollution_button->info_label, cbuf);
-        clear_wflag(order_pollution_button, WF_HIDDEN);
-      } else {
-        set_wflag(order_pollution_button, WF_HIDDEN);
-      }
-
       if (can_unit_paradrop(&(wld.map), punit)) {
         local_show(ID_UNIT_ORDER_PARADROP);
       } else {
         local_hide(ID_UNIT_ORDER_PARADROP);
       }
 
-      pextra = prev_extra_in_tile(ptile, ERM_CLEANFALLOUT,
-                                  unit_owner(punit), punit);
+      pextra = prev_cleanable_in_tile(ptile, unit_owner(punit), punit);
+
       if (pextra != NULL
-          && can_unit_do_activity_targeted_client(punit, ACTIVITY_FALLOUT, pextra)) {
-        time = turns_to_activity_done(ptile, ACTIVITY_FALLOUT, pextra,
-                                      punit);
-        /* TRANS: "Clean Fallout (N) 3 turns" */
+          && can_unit_do_activity_targeted_client(punit, ACTIVITY_CLEAN,
+                                                  pextra)) {
+        time = turns_to_activity_done(ptile, ACTIVITY_CLEAN,
+                                      pextra, punit);
+        /* TRANS: "Clean Pollution (P) 3 turns" */
         fc_snprintf(cbuf, sizeof(cbuf), _("Clean %s (%s) %d %s"),
-                    extra_name_translation(pextra), "N", time,
+                    extra_name_translation(pextra), "P", time,
                     PL_("turn", "turns", time));
-        copy_chars_to_utf8_str(order_fallout_button->info_label, cbuf);
-        clear_wflag(order_fallout_button, WF_HIDDEN);
+        copy_chars_to_utf8_str(order_clean_button->info_label, cbuf);
+        clear_wflag(order_clean_button, WF_HIDDEN);
       } else {
-        set_wflag(order_fallout_button, WF_HIDDEN);
+        set_wflag(order_clean_button, WF_HIDDEN);
       }
 
       if (can_unit_do_activity_client(punit, ACTIVITY_SENTRY)) {
