@@ -183,7 +183,7 @@ static int redraw_edit(struct widget *edit_widget)
     /* set position and blit text */
     if (text) {
       rdest.y += (pedit->h - text->h) / 2;
-      /* blit centred text to botton */
+      /* blit centred text to bottom */
       if (edit_widget->string_utf8->style & SF_CENTER) {
         rdest.x += (pedit->w - text->w) / 2;
       } else {
@@ -420,20 +420,20 @@ int draw_edit(struct widget *pedit, Sint16 start_x, Sint16 start_y)
   NOTE: This functions can return NULL in 'edit_widget->string_utf8->text' but
         never free 'edit_widget->string_utf8' struct.
 **************************************************************************/
-static Uint16 edit_key_down(SDL_Keysym key, void *data)
+static widget_id edit_key_down(SDL_Keysym key, void *data)
 {
   struct text_edit *edt = (struct text_edit *)data;
   struct utf8_char *input_chain_tmp;
   bool redraw = FALSE;
 
-  /* find which key is pressed */
+  /* Find which key is pressed */
   switch (key.sym) {
     case SDLK_ESCAPE:
-      /* exit from loop without changes */
+      /* Exit from loop without changes */
       return ED_ESC;
     case SDLK_RETURN:
     case SDLK_KP_ENTER:
-      /* exit from loop */
+      /* Exit from loop */
       return ED_RETURN;
       /*
     case SDLK_KP6:
@@ -574,7 +574,7 @@ static Uint16 edit_key_down(SDL_Keysym key, void *data)
     break;
   default:
     break;
-  } /* key pressed switch */
+  } /* Key pressed switch */
 
   if (redraw) {
     redraw_edit_chain(edt);
@@ -586,7 +586,7 @@ static Uint16 edit_key_down(SDL_Keysym key, void *data)
 /**********************************************************************//**
   Handle textinput strings coming to the edit widget
 **************************************************************************/
-static Uint16 edit_textinput(const char *text, void *data)
+static widget_id edit_textinput(const char *text, void *data)
 {
   struct text_edit *edt = (struct text_edit *)data;
   struct utf8_char *input_chain_tmp;
@@ -656,8 +656,8 @@ static Uint16 edit_textinput(const char *text, void *data)
 /**********************************************************************//**
   Handle mouse down events on edit widget.
 **************************************************************************/
-static Uint16 edit_mouse_button_down(SDL_MouseButtonEvent *button_event,
-                                     void *data)
+static widget_id edit_mouse_button_down(SDL_MouseButtonEvent *button_event,
+                                        void *data)
 {
   struct text_edit *edt = (struct text_edit *)data;
 
@@ -666,12 +666,12 @@ static Uint16 edit_mouse_button_down(SDL_MouseButtonEvent *button_event,
           && button_event->x < edt->pwidget->size.x + edt->bg->w
           && button_event->y >= edt->pwidget->size.y
           && button_event->y < edt->pwidget->size.y + edt->bg->h)) {
-      /* exit from loop */
-      return (Uint16)ED_MOUSE;
+      /* Exit from loop */
+      return (widget_id)ED_MOUSE;
     }
   }
 
-  return (Uint16)ID_ERROR;
+  return (widget_id)ID_ERROR;
 }
 
 /**********************************************************************//**
@@ -713,7 +713,7 @@ enum edit_return_codes edit_field(struct widget *edit_widget)
   edt.end_text_chain->next = NULL;
   edt.end_text_chain->prev = NULL;
 
-  /* set font style (if any ) */
+  /* Set font style (if any ) */
   if (!((edit_widget->string_utf8->style & 0x0F) & TTF_STYLE_NORMAL)) {
     TTF_SetFontStyle(edit_widget->string_utf8->font,
                      (edit_widget->string_utf8->style & 0x0F));
@@ -724,7 +724,7 @@ enum edit_return_codes edit_field(struct widget *edit_widget)
                              edt.end_text_chain->chr,
                              edit_widget->string_utf8->fgcol);
 
-  /* create surface for each font in chain and find chain length */
+  /* Create surface for each font in chain and find chain length */
   if (edt.begin_text_chain) {
     input_chain_tmp = edt.begin_text_chain;
 
@@ -765,10 +765,10 @@ enum edit_return_codes edit_field(struct widget *edit_widget)
   
   set_wstate(edit_widget, FC_WS_PRESSED);
   {
-    /* local loop */  
-    Uint16 rety = gui_event_loop((void *)&edt, NULL,
-                                 edit_key_down, NULL, edit_textinput, NULL, NULL, NULL,
-                                 edit_mouse_button_down, NULL, NULL);
+    /* Local loop */  
+    widget_id rety = gui_event_loop((void *)&edt, NULL,
+                                    edit_key_down, NULL, edit_textinput, NULL, NULL, NULL,
+                                    edit_mouse_button_down, NULL, NULL);
 
     if (edt.begin_text_chain == edt.end_text_chain) {
       edt.begin_text_chain = NULL;
