@@ -427,7 +427,7 @@ int blit_entire_src(SDL_Surface *psrc, SDL_Surface *pdest,
   Return the pixel value at (x, y)
   NOTE: The surface must be locked before calling this!
 **************************************************************************/
-Uint32 get_pixel(SDL_Surface *surf, Sint16 x, Sint16 y)
+Uint32 get_pixel(SDL_Surface *surf, int x, int y)
 {
   if (!surf) {
     return 0x0;
