@@ -24,6 +24,7 @@
 #include "citydlg_g.h"
 #include "connectdlg_g.h"
 #include "dialogs_g.h"
+#include "diplodlg_g.h"
 #include "editgui_g.h"
 #include "graphics_g.h"
 #include "gui_main_g.h"
@@ -120,9 +121,9 @@ void tileset_type_set(enum ts_type type)
 /**********************************************************************//**
   Call load_gfxfile callback
 **************************************************************************/
-struct sprite *load_gfxfile(const char *filename)
+struct sprite *load_gfxfile(const char *filename, bool svgflag)
 {
-  return funcs.load_gfxfile(filename);
+  return funcs.load_gfxfile(filename, svgflag);
 }
 
 /**********************************************************************//**
@@ -257,6 +258,18 @@ void canvas_put_sprite_full(struct canvas *pcanvas,
   funcs.canvas_put_sprite_full(pcanvas, canvas_x, canvas_y, psprite);
 }
 
+/**********************************************************************//**
+  Call canvas_put_sprite_full_scaled callback
+**************************************************************************/
+void canvas_put_sprite_full_scaled(struct canvas *pcanvas,
+                                   int canvas_x, int canvas_y,
+                                   int canvas_w, int canvas_h,
+                                   struct sprite *psprite)
+{
+  funcs.canvas_put_sprite_full_scaled(pcanvas, canvas_x, canvas_y,
+                                      canvas_w, canvas_h, psprite);
+}
+
 /**********************************************************************//**
   Call canvas_put_sprite_fogged callback
 **************************************************************************/
@@ -637,3 +650,70 @@ char **get_usable_themes_in_directory(const char *directory, int *count)
 {
   return funcs.get_usable_themes_in_directory(directory, count);
 }
+
+/**********************************************************************//**
+  Call gui_init_meeting callback
+**************************************************************************/
+void gui_init_meeting(struct Treaty *ptreaty, struct player *they,
+                      struct player *initiator)
+{
+  funcs.gui_init_meeting(ptreaty, they, initiator);
+}
+
+/**********************************************************************//**
+  Call gui_recv_cancel_meeting callback
+**************************************************************************/
+void gui_recv_cancel_meeting(struct Treaty *ptreaty, struct player *they,
+                             struct player *initiator)
+{
+  funcs.gui_recv_cancel_meeting(ptreaty, they, initiator);
+}
+
+/**********************************************************************//**
+  Call gui_prepare_clause_updt callback
+**************************************************************************/
+void gui_prepare_clause_updt(struct Treaty *ptreaty, struct player *they)
+{
+  funcs.gui_prepare_clause_updt(ptreaty, they);
+}
+
+/**********************************************************************//**
+  Call gui_recv_create_clause callback
+**************************************************************************/
+void gui_recv_create_clause(struct Treaty *ptreaty, struct player *they)
+{
+  funcs.gui_recv_create_clause(ptreaty, they);
+}
+
+/**********************************************************************//**
+  Call gui_recv_remove_clause callback
+**************************************************************************/
+void gui_recv_remove_clause(struct Treaty *ptreaty, struct player *they)
+{
+  funcs.gui_recv_remove_clause(ptreaty, they);
+}
+
+/**********************************************************************//**
+  Call gui_recv_accept_treaty callback
+**************************************************************************/
+void gui_recv_accept_treaty(struct Treaty *ptreaty, struct player *they)
+{
+  funcs.gui_recv_accept_treaty(ptreaty, they);
+}
+
+/**********************************************************************//**
+  Call request_action_confirmation callback
+**************************************************************************/
+void request_action_confirmation(const char *expl,
+                                 struct act_confirmation_data *data)
+{
+  funcs.request_action_confirmation(expl, data);
+}
+
+/**********************************************************************//**
+  Call popup_image callback
+**************************************************************************/
+void popup_image(const char *tag)
+{
+  funcs.popup_image(tag);
+}
