@@ -164,7 +164,7 @@ unit_attack_unit_at_tile_result(const struct unit *punit,
   /* 3. Most units can not attack non-native terrain.
    *    Most ships can attack land tiles (shore bombardment) */
   if (!is_native_tile(unit_type_get(punit), dest_tile)
-      && !can_attack_non_native(unit_type_get(punit))) {
+      && !can_attack_non_native_hard_reqs(unit_type_get(punit))) {
     return ATT_NONNATIVE_DST;
   }
 
@@ -264,6 +264,46 @@ unit_attack_units_at_tile_result(const struct unit *punit,
   }
 }
 
+/*******************************************************************//**
+  Check if unit can wipe unit stack from tile.
+***********************************************************************/
+enum unit_attack_result
+unit_wipe_units_at_tile_result(const struct unit *punit,
+                               const struct tile *ptile)
+{
+  /* Can unit wipe in general? */
+  if (!utype_can_do_action(unit_type_get(punit), ACTION_WIPE_UNITS)) {
+    return ATT_NON_ATTACK;
+  }
+
+  unit_list_iterate(ptile->units, target) {
+    if (get_total_defense_power(punit, target) > 0) {
+      return ATT_NOT_WIPABLE;
+    }
+
+    /* Can't wipe with ground unit from ocean, except for marines */
+    if (!is_native_tile(unit_type_get(punit), unit_tile(punit))
+        && !utype_can_do_act_when_ustate(unit_type_get(punit), ACTION_WIPE_UNITS,
+                                         USP_NATIVE_TILE, FALSE)) {
+      return ATT_NONNATIVE_SRC;
+    }
+
+    /* Most units can not wipe on non-native terrain.
+     * Most ships can wipe on land tiles (shore bombardment) */
+    if (!is_native_tile(unit_type_get(punit), ptile)
+        && !can_attack_non_native_hard_reqs(unit_type_get(punit))) {
+      return ATT_NONNATIVE_DST;
+    }
+
+    /* Only fighters can wipe planes, except in city or airbase attacks */
+    if (!is_unit_reachable_at(target, punit, ptile)) {
+      return ATT_UNREACHABLE;
+    }
+  } unit_list_iterate_end;
+
+  return ATT_OK;
+}
+
 /*******************************************************************//**
   Is unit (1) diplomatically allowed to attack and (2) physically able
   to do so?
@@ -437,10 +477,11 @@ void get_modified_firepower(const struct civ_map *nmap,
 ***********************************************************************/
 double unit_win_chance(const struct civ_map *nmap,
                        const struct unit *attacker,
-		       const struct unit *defender)
+                       const struct unit *defender,
+                       const struct action *paction)
 {
   int def_power = get_total_defense_power(attacker, defender);
-  int att_power = get_total_attack_power(attacker, defender);
+  int att_power = get_total_attack_power(attacker, defender, paction);
   double chance;
   int def_fp, att_fp;
 
@@ -567,14 +608,26 @@ static int get_defense_power(const struct unit *punit)
   Return the modified attack power of a unit.
 ***********************************************************************/
 int get_total_attack_power(const struct unit *attacker,
-                           const struct unit *defender)
+                           const struct unit *defender,
+                           const struct action *paction)
 {
   int mod;
   int attackpower = get_attack_power(attacker);
+  struct tile *deftile = unit_tile(defender);
+  struct city *pcity = tile_city(deftile);
 
-  mod = 100 + get_unittype_bonus(unit_owner(attacker), unit_tile(defender),
-                                 unit_type_get(attacker), NULL,
-                                 EFT_ATTACK_BONUS);
+  /* Note: get_unit_bonus() would use the tile unit is on,
+   *       but we want defending tile. */
+  mod = 100 + get_target_bonus_effects(NULL,
+                                       &(const struct req_context) {
+                                         .player = unit_owner(attacker),
+                                         .city = pcity,
+                                         .tile = deftile,
+                                         .unit = attacker,
+                                         .unittype = unit_type_get(attacker),
+                                         .action = paction,
+                                       },
+                                       NULL, EFT_ATTACK_BONUS);
 
   return attackpower * mod / 100;
 }
@@ -782,7 +835,8 @@ static int get_defense_rating(const struct civ_map *nmap,
 ***********************************************************************/
 struct unit *get_defender(const struct civ_map *nmap,
                           const struct unit *attacker,
-			  const struct tile *ptile)
+                          const struct tile *ptile,
+                          const struct action *paction)
 {
   struct unit *bestdef = NULL;
   int bestvalue = -99, best_cost = 0, rating_of_best = 0;
@@ -806,8 +860,9 @@ struct unit *get_defender(const struct civ_map *nmap,
       int build_cost = unit_build_shield_cost_base(defender);
       int defense_rating = get_defense_rating(nmap, attacker, defender);
       /* This will make units roughly evenly good defenders look alike. */
-      int unit_def 
-        = (int) (100000 * (1 - unit_win_chance(nmap, attacker, defender)));
+      int unit_def
+        = (int) (100000 * (1 - unit_win_chance(nmap, attacker,
+                                               defender, paction)));
 
       fc_assert_action(0 <= unit_def, continue);
 
@@ -861,7 +916,8 @@ struct unit *get_attacker(const struct civ_map *nmap,
     if (pplayers_allied(unit_owner(defender), unit_owner(attacker))) {
       return NULL;
     }
-    unit_a = (int) (100000 * (unit_win_chance(nmap, attacker, defender)));
+    unit_a = (int) (100000 * (unit_win_chance(nmap, attacker,
+                                              defender, NULL)));
     if (unit_a > bestvalue
         || (unit_a == bestvalue && build_cost < best_cost)) {
       bestvalue = unit_a;
@@ -876,14 +932,17 @@ struct unit *get_attacker(const struct civ_map *nmap,
 /**********************************************************************//**
   Returns the defender of the tile in a diplomatic battle or NULL if no
   diplomatic defender could be found.
+
   @param act_unit the diplomatic attacker, trying to perform an action.
   @param pvictim  unit that should be excluded as a defender.
   @param tgt_tile the tile to defend.
+  @param paction  action that the attacker performs.
   @return the defender or NULL if no diplomatic defender could be found.
 **************************************************************************/
 struct unit *get_diplomatic_defender(const struct unit *act_unit,
                                      const struct unit *pvictim,
-                                     const struct tile *tgt_tile)
+                                     const struct tile *tgt_tile,
+                                     const struct action *paction)
 {
   fc_assert_ret_val(act_unit, NULL);
   fc_assert_ret_val(tgt_tile, NULL);
