@@ -41,6 +41,12 @@
 
 static bool initialized = FALSE;
 
+struct user_effect {
+  enum effect_type ai_value_as;
+};
+
+struct user_effect ueffects[EFT_USER_EFFECT_LAST + 1 - EFT_USER_EFFECT_1];
+
 /**************************************************************************
   The code creates a ruleset cache on ruleset load. This constant cache
   is used to speed up effects queries. After the cache is created it is
@@ -62,6 +68,7 @@ static bool initialized = FALSE;
   * get_city_output_bonus()
   * get_city_tile_output_bonus()
   * get_city_specialist_output_bonus()
+  * get_tile_bonus()
   * get_tile_output_bonus()
   * get_building_bonus()
   * get_unittype_bonus()
@@ -143,7 +150,7 @@ struct effect_list *get_effects(enum effect_type effect_type)
 
   Currently only buildings, advances, and governments are supported.
 **************************************************************************/
-struct effect_list *get_req_source_effects(struct universal *psource)
+struct effect_list *get_req_source_effects(const struct universal *psource)
 {
   int type, value;
 
@@ -194,13 +201,31 @@ struct effect *effect_new(enum effect_type type, int value,
   effect_list_append(get_effects(type), peffect);
 
   /* Only relevant for ruledit and other rulesave users. */
-  peffect->ruledit_do_not_save = FALSE;
+  peffect->rulesave.do_not_save = FALSE;
+  peffect->rulesave.comment = NULL;
 
   return peffect;
 }
 
+/**********************************************************************//**
+  Free resources reserved for the effect.
+
+  This does not remove effect from the cache lists.
+  See effect_remove() for that.
+**************************************************************************/
+void effect_free(struct effect *peffect)
+{
+  requirement_vector_free(&peffect->reqs);
+  if (peffect->rulesave.comment != NULL) {
+    free(peffect->rulesave.comment);
+  }
+  free(peffect);
+}
+
 /**********************************************************************//**
   Remove effect from the caches.
+
+  This does not free effect itself. See effect_free() for that.
 **************************************************************************/
 void effect_remove(struct effect *peffect)
 {
@@ -209,12 +234,24 @@ void effect_remove(struct effect *peffect)
 }
 
 /**********************************************************************//**
-  Return new copy of the effect
+  Create copy of the effect. It gets fully registered to
+  the ruleset caches.
+
+  @param old           Original effect to copy
+  @param override_type Type of the effect to create, or effect_type_invalid()
+                       to copy type from the original effect
+
+  @return              Newly created effect
 **************************************************************************/
-struct effect *effect_copy(struct effect *old)
+struct effect *effect_copy(struct effect *old,
+                           enum effect_type override_type)
 {
-  struct effect *new_eff = effect_new(old->type, old->value,
-                                      old->multiplier);
+  struct effect *new_eff;
+  enum effect_type type = (effect_type_is_valid(override_type)
+                           ? override_type
+                           : old->type);
+
+  new_eff = effect_new(type, old->value, old->multiplier);
 
   requirement_vector_iterate(&old->reqs, preq) {
     effect_req_append(new_eff, *preq);
@@ -232,9 +269,24 @@ void effect_req_append(struct effect *peffect, struct requirement req)
 
   requirement_vector_append(&peffect->reqs, req);
 
-  if (eff_list) {
+  if (eff_list != NULL) {
     effect_list_append(eff_list, peffect);
   }
+
+  if (req.source.kind == VUT_IMPR_FLAG) {
+    improvement_iterate(impr) {
+      if (improvement_has_flag(impr, req.source.value.impr_flag)) {
+        eff_list = get_req_source_effects(&(const struct universal) {
+                                            .kind = VUT_IMPROVEMENT,
+                                            .value.building = impr
+                                          });
+
+        if (eff_list != NULL) {
+          effect_list_append(eff_list, peffect);
+        }
+      }
+    } improvement_iterate_end;
+  }
 }
 
 /**********************************************************************//**
@@ -262,11 +314,17 @@ void ruleset_cache_init(void)
   for (i = 0; i < ARRAY_SIZE(ruleset_cache.reqs.advances); i++) {
     ruleset_cache.reqs.advances[i] = effect_list_new();
   }
+
+  /* By default, user effects are valued as themselves
+   * (currently meaning that they get no value at all) */
+  for (i = EFT_USER_EFFECT_1 ; i <= EFT_USER_EFFECT_LAST; i++) {
+    ueffects[USER_EFFECT_NUMBER(i)].ai_value_as = i;
+  }
 }
 
 /**********************************************************************//**
-  Free the ruleset cache.  This should be called at the end of the game or
-  when the client disconnects from the server.  See ruleset_cache_init.
+  Free the ruleset cache. This should be called at the end of the game or
+  when the client disconnects from the server. See ruleset_cache_init().
 **************************************************************************/
 void ruleset_cache_free(void)
 {
@@ -275,8 +333,7 @@ void ruleset_cache_free(void)
 
   if (tracker_list) {
     effect_list_iterate(tracker_list, peffect) {
-      requirement_vector_free(&peffect->reqs);
-      free(peffect);
+      effect_free(peffect);
     } effect_list_iterate_end;
     effect_list_destroy(tracker_list);
     ruleset_cache.tracker = NULL;
@@ -801,6 +858,27 @@ int get_city_bonus(const struct city *pcity, enum effect_type effect_type)
                                   NULL, effect_type);
 }
 
+/**********************************************************************//**
+  Returns the effect bonus at a tile.
+  Even if the tile has a city, player requirements are not about city owner
+  but tile owner.
+**************************************************************************/
+int get_tile_bonus(const struct tile *ptile, enum effect_type effect_type)
+{
+  if (!initialized) {
+    return 0;
+  }
+
+  return get_target_bonus_effects(NULL,
+                                  &(const struct req_context) {
+                                    .player = tile_owner(ptile),
+                                    .city = tile_city(ptile),
+                                    .tile = ptile,
+                                  },
+                                  NULL,
+                                  effect_type);
+}
+
 /**********************************************************************//**
   Returns the effect bonus of a specialist in a city.
 **************************************************************************/
@@ -828,8 +906,8 @@ int get_city_specialist_output_bonus(const struct city *pcity,
   pcity must be supplied.
 
   FIXME: this is now used both for tile bonuses, tile-output bonuses,
-  and city-output bonuses.  Thus ptile or poutput may be NULL for
-  certain callers.  This could be changed by adding 2 new functions to
+  and city-output bonuses. Thus ptile or poutput may be NULL for
+  certain callers. This could be changed by adding 2 new functions to
   the interface but they'd be almost identical and their likely names
   would conflict with functions already in city.c.
   It's also very similar to get_tile_output_bonus(); it should be
@@ -1254,3 +1332,42 @@ bool iterate_effect_cache(iec_cb cb, void *data)
 
   return TRUE;
 }
+
+/**********************************************************************//**
+  Is the effect type an user effect type?
+**************************************************************************/
+bool is_user_effect(enum effect_type eff)
+{
+  return eff >= EFT_USER_EFFECT_1 && eff <= EFT_USER_EFFECT_LAST;
+}
+
+/**********************************************************************//**
+  Set the ai_valued_as effect type for the target effect.
+  Target must be an user effect.
+**************************************************************************/
+void user_effect_ai_valued_set(enum effect_type tgt, enum effect_type valued_as)
+{
+  fc_assert(is_user_effect(tgt));
+
+  ueffects[USER_EFFECT_NUMBER(tgt)].ai_value_as = valued_as;
+}
+
+/**********************************************************************//**
+  Get the ai_valued_as effect type for effect. Can be used also
+  for non-user effects - then it just returns the effect itself.
+**************************************************************************/
+enum effect_type user_effect_ai_valued_as(enum effect_type real)
+{
+  /* Client doesn't know these. */
+  fc_assert(is_server());
+
+  if (!is_user_effect(real)) {
+    return real;
+  }
+
+  if (!is_user_effect(ueffects[USER_EFFECT_NUMBER(real)].ai_value_as)) {
+    return ueffects[USER_EFFECT_NUMBER(real)].ai_value_as;
+  }
+
+  return real;
+}
