@@ -17,24 +17,24 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#include <time.h>	/* time_t */
+#include <time.h>       /* time_t */
 
 #ifdef FREECIV_HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 
 /* utility */
-#include "fcthread.h"
 #include "randseed.h"
 #include "shared.h"
 #include "timing.h"
 
 /* common */
-#include "connection.h"		/* struct conn_list */
+#include "connection.h"         /* struct conn_list */
 #include "fc_types.h"
-#include "player.h"
 #include "packets.h"
-#include "world_object.h"
+#include "traits.h"
+
+struct world;
 
 enum debug_globals {
   DEBUG_FERRIES,
@@ -90,17 +90,20 @@ struct civ_game {
   struct packet_calendar_info calendar;
   struct packet_timeout_info tinfo;
 
-  struct government *default_government; /* may be overridden by nation */
+  struct government *default_government; /* May be overridden by nation */
   struct government *government_during_revolution;
 
-  struct conn_list *all_connections;   /* including not yet established */
-  struct conn_list *est_connections;   /* all established client conns */
-  struct conn_list *glob_observers;    /* global observers */
+  struct conn_list *all_connections;   /* Including not yet established */
+  struct conn_list *est_connections;   /* All established client conns */
+  struct conn_list *glob_observers;    /* Global observers */
+  struct conn_list *web_client_connections; /* Connections from web client */
 
-  struct veteran_system *veteran; /* veteran system */
+  struct veteran_system *veteran; /* Veteran system */
 
   struct rgbcolor *plr_bg_color;
 
+  int lua_timeout;
+
   struct {
     /* Items given to all players at game start.
      * Client gets this info for help purposes only. */
@@ -114,6 +117,8 @@ struct civ_game {
 
       bool ruleset_init;
       bool ruleset_ready;
+
+      bool fog_of_war;
     } client;
 
     struct {
@@ -150,7 +155,7 @@ struct civ_game {
       int incite_unit_factor;
       int init_vis_radius_sq;
       int kick_time;
-      int killunhomed;    /* slowly killing unhomed units */
+      int killunhomed;    /* Slowly killing unhomed units */
       int maxconnectionsperhost;
       int max_players;
       char nationset[MAX_LEN_NAME];
@@ -175,7 +180,7 @@ struct civ_game {
       int razechance;
       unsigned revealmap;
       int revolution_length;
-      int spaceship_travel_time;
+      int spaceship_travel_pct;
       bool threaded_save;
       int save_compress_level;
       enum fz_method save_compress_type;
@@ -194,17 +199,17 @@ struct civ_game {
       int tcptimeout;
       int techpenalty;
       bool turnblock;
-      int unitwaittime;   /* minimal time between two movements of a unit */
+      int unitwaittime;   /* Minimal time between two movements of a unit */
       int upgrade_veteran_loss;
       bool vision_reveal_tiles;
 
       bool debug[DEBUG_LAST];
-      int timeoutint;     /* increase timeout every N turns... */
+      int timeoutint;     /* Increase timeout every N turns... */
       int timeoutinc;     /* ... by this amount ... */
       int timeoutincmult; /* ... and multiply timeoutinc by this amount ... */
       int timeoutintinc;  /* ... and increase timeoutint by this amount */
       int timeoutcounter; /* timeoutcounter - timeoutint = turns to next inc. */
-      int timeoutaddenemymove; /* minimum timeout after an enemy move is seen */
+      int timeoutaddenemymove; /* Minimum timeout after an enemy move is seen */
 
       time_t last_ping;
       struct timer *phase_timer;   /* Time since seconds_to_phase_done was set. */
@@ -217,10 +222,11 @@ struct civ_game {
       float turn_change_time;
       char connectmsg[MAX_LEN_MSG];
       char save_name[MAX_LEN_NAME];
+      char orig_game_version[MAX_LEN_NAME];
       bool scorelog;
       enum scorelog_level scoreloglevel;
-      char scorefile[MAX_LEN_NAME];
-      int scoreturn;    /* next make_history_report() */
+      char scorefile[MAX_LEN_PATH];
+      int scoreturn;    /* Next make_history_report() */
       randseed seed_setting;
       randseed seed;
 
@@ -229,7 +235,7 @@ struct civ_game {
       bool nuclear_winter;
       int nuclear_winter_percent;
 
-      bool fogofwar_old; /* as the fog_of_war bit get changed by setting
+      bool fogofwar_old; /* As the fog_of_war bit get changed by setting
                           * the server we need to remember the old setting */
       bool last_updated_year; /* last_updated is still counted as year in this
                                * game. */
@@ -242,6 +248,7 @@ struct civ_game {
       struct rgbcolor_list *plr_colors;
 
       struct section_file *luadata;
+      int dbid;
 
       struct {
         int turns;
@@ -250,11 +257,11 @@ struct civ_game {
         bool info;
       } event_cache;
 
-      /* used by the map editor to control game_save. */
+      /* Used by the map editor to control game_save. */
       struct {
-        bool save_known; /* loading will just reveal the squares around
+        bool save_known; /* Loading will just reveal the squares around
                           * cities and units */
-        bool save_starts; /* start positions will be auto generated */
+        bool save_starts; /* Start positions will be auto generated */
         bool save_private_map; /* FoW map; will be created if not saved */
       } save_options;
 
@@ -269,6 +276,10 @@ struct civ_game {
 
       struct trait_limits default_traits[TRAIT_COUNT];
 
+      struct player *random_move_time;
+
+      char default_ai_type_name[256];
+
       struct {
         char *description_file;
         char *nationlist;
@@ -294,6 +305,20 @@ struct civ_game {
   } callbacks;
 };
 
+extern struct civ_game game;
+
+#ifdef FREECIV_WEB
+/**********************************************************************//**
+  Is there currently any connections from web-client(s)
+**************************************************************************/
+static inline bool any_web_conns(void)
+{
+  return conn_list_size(game.web_client_connections) > 0;
+}
+#else  /* FREECIV_WEB */
+#define any_web_conns() (FALSE)
+#endif /* FREECIV_WEB */
+
 void game_init(bool keep_ruleset_value);
 void game_map_init(void);
 void game_free(void);
@@ -342,9 +367,6 @@ static inline bool is_ruleset_compat_mode(void)
   return _ruleset_compat_mode;
 }
 
-extern struct civ_game game;
-extern struct world wld;
-
 #define GAME_DEFAULT_SEED        0
 #define GAME_MIN_SEED            0
 #define GAME_MAX_SEED            (MAX_UINT32 >> 1)
@@ -503,7 +525,7 @@ extern struct world wld;
 
 #define GAME_DEFAULT_MGR_FOODNEEDED   TRUE
 
-/* definition of the migration distance in relation to the current city
+/* Definition of the migration distance in relation to the current city
  * radius; 0 means that migration is possible if the second city is within
  * the city radius */
 #define GAME_DEFAULT_MGR_DISTANCE     0
@@ -562,9 +584,9 @@ extern struct world wld;
 #define GAME_DEFAULT_VICTORY_CONDITIONS (1 << VC_SPACERACE | 1 << VC_ALLIED)
 #define GAME_DEFAULT_END_SPACESHIP   TRUE
 
-#define GAME_DEFAULT_SPACESHIP_TRAVEL_TIME 100
-#define GAME_MIN_SPACESHIP_TRAVEL_TIME     50
-#define GAME_MAX_SPACESHIP_TRAVEL_TIME     1000
+#define GAME_DEFAULT_SPACESHIP_TRAVEL_PCT 100
+#define GAME_MIN_SPACESHIP_TRAVEL_PCT     50
+#define GAME_MAX_SPACESHIP_TRAVEL_PCT     1000
 
 #define GAME_DEFAULT_TURNBLOCK       TRUE
 
@@ -676,6 +698,10 @@ extern struct world wld;
 #define GAME_HARDCODED_DEFAULT_SKILL_LEVEL 3 /* that was 'easy' in old saves */
 #define GAME_OLD_DEFAULT_SKILL_LEVEL 5  /* normal; for oldest save games */
 
+#define GAME_DEFAULT_TOP_CITIES_COUNT 5
+#define GAME_MIN_TOP_CITIES_COUNT     0
+#define GAME_MAX_TOP_CITIES_COUNT     40
+
 #define GAME_DEFAULT_DEMOGRAPHY      "NASRLPEMOCqrb"
 #define GAME_DEFAULT_ALLOW_TAKE      "HAhadOo"
 
@@ -695,7 +721,9 @@ extern struct world wld;
 #define GAME_MIN_COMPRESS_LEVEL     1
 #define GAME_MAX_COMPRESS_LEVEL     9
 
-#if defined(FREECIV_HAVE_LIBLZMA)
+#if defined(FREECIV_HAVE_LIBZSTD)
+#  define GAME_DEFAULT_COMPRESS_TYPE FZ_ZSTD
+#elif defined(FREECIV_HAVE_LIBLZMA)
 #  define GAME_DEFAULT_COMPRESS_TYPE FZ_XZ
 #elif defined(FREECIV_HAVE_LIBZ)
 #  define GAME_DEFAULT_COMPRESS_TYPE FZ_ZLIB
@@ -717,13 +745,17 @@ extern struct world wld;
 #define GAME_DEFAULT_AIRLIFTINGSTYLE AIRLIFTING_CLASSICAL
 #define GAME_DEFAULT_PERSISTENTREADY PERSISTENTR_DISABLED
 
-#define GAME_MAX_READ_RECURSION 10 /* max recursion for the read command */
+#define GAME_MAX_READ_RECURSION 10 /* Max recursion for the read command */
 
 #define GAME_DEFAULT_KICK_TIME 1800     /* 1800 seconds = 30 minutes. */
 #define GAME_MIN_KICK_TIME 0            /* 0 = disabling. */
 #define GAME_MAX_KICK_TIME 86400        /* 86400 seconds = 24 hours. */
 
-/* Max distance from the capital used to calculat the bribe cost. */
+#define GAME_DEFAULT_LUA_TIMEOUT 5
+#define GAME_MIN_LUA_TIMEOUT     5
+#define GAME_MAX_LUA_TIMEOUT     30
+
+/* Max distance from the capital used to calculate the bribe cost. */
 #define GAME_UNIT_BRIBE_DIST_MAX 32
 
 /* Max number of recursive transports. */
@@ -847,13 +879,13 @@ extern struct world wld;
 #define RS_MIN_UPGRADE_VETERAN_LOSS              0
 #define RS_MAX_UPGRADE_VETERAN_LOSS              MAX_VET_LEVELS
 
-#define RS_DEFAULT_TECH_UPKEEP_DIVIDER   100
-#define RS_MIN_TECH_UPKEEP_DIVIDER       1
-#define RS_MAX_TECH_UPKEEP_DIVIDER       100000
+#define RS_DEFAULT_TECH_UPKEEP_DIVIDER           100
+#define RS_MIN_TECH_UPKEEP_DIVIDER               1
+#define RS_MAX_TECH_UPKEEP_DIVIDER               100000
 
-#define RS_DEFAULT_BASE_TECH_COST                20
-#define RS_MIN_BASE_TECH_COST                    0
-#define RS_MAX_BASE_TECH_COST                    20000
+#define RS_DEFAULT_HP                            10
+#define RS_MIN_HP                                1
+#define RS_MAX_HP                                10000
 
 #define RS_DEFAULT_POISON_EMPTIES_FOOD_STOCK     FALSE
 #define RS_DEFAULT_STEAL_MAP_REVEALS_CITIES      TRUE
@@ -861,6 +893,8 @@ extern struct world wld;
 #define RS_DEFAULT_USER_ACTION_TARGET_KIND       ATK_UNIT
 #define RS_DEFAULT_ACTION_MIN_RANGE              0
 #define RS_DEFAULT_ACTION_MAX_RANGE              1
+/* Can't move less than one step, or it's not move at all */
+#define RS_DEFAULT_MOVE_MIN_RANGE                1
 #define RS_DEFAULT_EXPLODE_NUCLEAR_MAX_RANGE     0
 
 #define RS_ACTION_NO_MAX_DISTANCE                "unlimited"
@@ -869,4 +903,4 @@ extern struct world wld;
 }
 #endif /* __cplusplus */
 
-#endif  /* FC__GAME_H */
+#endif /* FC__GAME_H */
