@@ -368,6 +368,8 @@ struct packet_city_info {
   bool did_sell;
   bool was_happy;
   bool had_famine;
+  int anarchy;
+  int rapture;
   bool diplomat_investigate;
   int walls;
   int city_image;
@@ -534,7 +536,8 @@ struct packet_player_info {
   int science;
   int luxury;
   int infrapoints;
-  int tech_upkeep;
+  int tech_upkeep_16;
+  int tech_upkeep_32;
   int science_cost;
   bool is_connected;
   int revolution_finishes;
