@@ -18,13 +18,12 @@
 extern "C" {
 #endif /* __cplusplus */
 
-/* utility */
-#include "astring.h"
-
 /* common */
 #include "fc_types.h"
 #include "map_types.h"
 
+struct astring;
+
 /* Range of requirements.
  * Used in the network protocol.
  * Order is important -- wider ranges should come later -- some code
@@ -38,25 +37,27 @@ extern "C" {
 #define SPECENUM_NAME req_range
 #define SPECENUM_VALUE0 REQ_RANGE_LOCAL
 #define SPECENUM_VALUE0NAME "Local"
-#define SPECENUM_VALUE1 REQ_RANGE_CADJACENT
-#define SPECENUM_VALUE1NAME "CAdjacent"
-#define SPECENUM_VALUE2 REQ_RANGE_ADJACENT
-#define SPECENUM_VALUE2NAME "Adjacent"
-#define SPECENUM_VALUE3 REQ_RANGE_CITY
-#define SPECENUM_VALUE3NAME "City"
-#define SPECENUM_VALUE4 REQ_RANGE_TRADE_ROUTE
+#define SPECENUM_VALUE1 REQ_RANGE_TILE
+#define SPECENUM_VALUE1NAME "Tile"
+#define SPECENUM_VALUE2 REQ_RANGE_CADJACENT
+#define SPECENUM_VALUE2NAME "CAdjacent"
+#define SPECENUM_VALUE3 REQ_RANGE_ADJACENT
+#define SPECENUM_VALUE3NAME "Adjacent"
+#define SPECENUM_VALUE4 REQ_RANGE_CITY
+#define SPECENUM_VALUE4NAME "City"
+#define SPECENUM_VALUE5 REQ_RANGE_TRADE_ROUTE
 /* FIXME: -> "Trade Route" */
-#define SPECENUM_VALUE4NAME "Traderoute"
-#define SPECENUM_VALUE5 REQ_RANGE_CONTINENT
-#define SPECENUM_VALUE5NAME "Continent"
-#define SPECENUM_VALUE6 REQ_RANGE_PLAYER
-#define SPECENUM_VALUE6NAME "Player"
-#define SPECENUM_VALUE7 REQ_RANGE_TEAM
-#define SPECENUM_VALUE7NAME "Team"
-#define SPECENUM_VALUE8 REQ_RANGE_ALLIANCE
-#define SPECENUM_VALUE8NAME "Alliance"
-#define SPECENUM_VALUE9 REQ_RANGE_WORLD
-#define SPECENUM_VALUE9NAME "World"
+#define SPECENUM_VALUE5NAME "Traderoute"
+#define SPECENUM_VALUE6 REQ_RANGE_CONTINENT
+#define SPECENUM_VALUE6NAME "Continent"
+#define SPECENUM_VALUE7 REQ_RANGE_PLAYER
+#define SPECENUM_VALUE7NAME "Player"
+#define SPECENUM_VALUE8 REQ_RANGE_TEAM
+#define SPECENUM_VALUE8NAME "Team"
+#define SPECENUM_VALUE9 REQ_RANGE_ALLIANCE
+#define SPECENUM_VALUE9NAME "Alliance"
+#define SPECENUM_VALUE10 REQ_RANGE_WORLD
+#define SPECENUM_VALUE10NAME "World"
 #define SPECENUM_COUNT REQ_RANGE_COUNT      /* Keep this last */
 #define REQ_RANGE_MAX (REQ_RANGE_COUNT - 1) /* REQ_RANGE_WORLD */
 #include "specenum_gen.h"
@@ -101,13 +102,15 @@ struct req_context {
   const struct city *city;
   const struct tile *tile;
 
-  /* for local-ranged requirements only */
+  /* For local-ranged requirements only */
   const struct unit *unit;
   const struct unit_type *unittype;
   const struct impr_type *building;
+  const struct extra_type *extra;
   const struct output_type *output;
   const struct specialist *specialist;
   const struct action *action;
+  const enum unit_activity activity;
 };
 
 enum req_unchanging_status {
@@ -172,7 +175,12 @@ bool are_reqs_active(const struct req_context *context,
                      const struct player *other_player,
                      const struct requirement_vector *reqs,
                      const enum   req_problem_type prob_type);
-
+bool are_reqs_active_ranges(const enum req_range min_range,
+                            const enum req_range max_range,
+                            const struct req_context *context,
+                            const struct player *other_player,
+                            const struct requirement_vector *reqs,
+                            const enum   req_problem_type prob_type);
 enum fc_tristate
 tri_req_active_turns(int pass, int period,
                      const struct req_context *context,
