@@ -39,6 +39,8 @@ struct civ_map;
  * Used in the network protocol. */
 #define U_LAST MAX_NUM_UNITS
 
+BV_DEFINE(bv_utypes, U_LAST);
+
 /* The largest distance a ruleset can allow a unit to paradrop.
  *
  * Remember to make sure that the field type of PACKET_RULESET_UNIT's
@@ -102,14 +104,27 @@ struct civ_map;
 #define SPECENUM_VALUE22 UCF_USER_FLAG_10
 #define SPECENUM_VALUE23 UCF_USER_FLAG_11
 #define SPECENUM_VALUE24 UCF_USER_FLAG_12
+#define SPECENUM_VALUE25 UCF_USER_FLAG_13
+#define SPECENUM_VALUE26 UCF_USER_FLAG_14
+#define SPECENUM_VALUE27 UCF_USER_FLAG_15
+#define SPECENUM_VALUE28 UCF_USER_FLAG_16
+#define SPECENUM_VALUE29 UCF_USER_FLAG_17
+#define SPECENUM_VALUE30 UCF_USER_FLAG_18
+#define SPECENUM_VALUE31 UCF_USER_FLAG_19
+#define SPECENUM_VALUE32 UCF_USER_FLAG_20
+#define SPECENUM_VALUE33 UCF_USER_FLAG_21
+#define SPECENUM_VALUE34 UCF_USER_FLAG_22
+#define SPECENUM_VALUE35 UCF_USER_FLAG_23
+#define SPECENUM_VALUE36 UCF_USER_FLAG_24
+#define SPECENUM_VALUE37 UCF_USER_FLAG_25
 
-/* keep this last */
+/* Keep this last */
 #define SPECENUM_COUNT UCF_COUNT
 #define SPECENUM_NAMEOVERRIDE
 #define SPECENUM_BITVECTOR bv_unit_class_flags
 #include "specenum_gen.h"
 
-#define UCF_LAST_USER_FLAG UCF_USER_FLAG_12
+#define UCF_LAST_USER_FLAG UCF_USER_FLAG_25
 #define MAX_NUM_USER_UCLASS_FLAGS (UCF_LAST_USER_FLAG                     \
                                    - UCF_USER_FLAG_1 + 1)
 
@@ -126,8 +141,6 @@ struct civ_map;
 /* Used in the network protocol. */
 BV_DEFINE(bv_unit_classes, UCL_LAST);
 
-enum hut_behavior { HUT_NORMAL, HUT_NOTHING, HUT_FRIGHTEN };
-
 enum move_level { MOVE_NONE, MOVE_PARTIAL, MOVE_FULL };
 
 struct extra_type_list;
@@ -159,11 +172,6 @@ struct unit_class {
     struct extra_type_list *hiding_extras;
     struct unit_class_list *subset_movers;
   } cache;
-
-  /* Used to upgrade the ruleset format version. */
-  struct {
-    enum hut_behavior hut_behavior;
-  } rscompat_cache_from_3_0;
 };
 
 /* Unit "special effects" flags:
@@ -264,55 +272,63 @@ struct unit_class {
 /* Can kill escaping units */
 #define SPECENUM_VALUE28 UTYF_CANKILLESCAPING
 #define SPECENUM_VALUE28NAME N_("?unitflag:CanKillEscaping")
+/* Makes random movement, not under player control */
+#define SPECENUM_VALUE29 UTYF_RANDOM_MOVEMENT
+#define SPECENUM_VALUE29NAME N_("?unitflag:RandomMovement")
 
-#define SPECENUM_VALUE29 UTYF_USER_FLAG_1
-#define SPECENUM_VALUE30 UTYF_USER_FLAG_2
-#define SPECENUM_VALUE31 UTYF_USER_FLAG_3
-#define SPECENUM_VALUE32 UTYF_USER_FLAG_4
-#define SPECENUM_VALUE33 UTYF_USER_FLAG_5
-#define SPECENUM_VALUE34 UTYF_USER_FLAG_6
-#define SPECENUM_VALUE35 UTYF_USER_FLAG_7
-#define SPECENUM_VALUE36 UTYF_USER_FLAG_8
-#define SPECENUM_VALUE37 UTYF_USER_FLAG_9
-#define SPECENUM_VALUE38 UTYF_USER_FLAG_10
-#define SPECENUM_VALUE39 UTYF_USER_FLAG_11
-#define SPECENUM_VALUE40 UTYF_USER_FLAG_12
-#define SPECENUM_VALUE41 UTYF_USER_FLAG_13
-#define SPECENUM_VALUE42 UTYF_USER_FLAG_14
-#define SPECENUM_VALUE43 UTYF_USER_FLAG_15
-#define SPECENUM_VALUE44 UTYF_USER_FLAG_16
-#define SPECENUM_VALUE45 UTYF_USER_FLAG_17
-#define SPECENUM_VALUE46 UTYF_USER_FLAG_18
-#define SPECENUM_VALUE47 UTYF_USER_FLAG_19
-#define SPECENUM_VALUE48 UTYF_USER_FLAG_20
-#define SPECENUM_VALUE49 UTYF_USER_FLAG_21
-#define SPECENUM_VALUE50 UTYF_USER_FLAG_22
-#define SPECENUM_VALUE51 UTYF_USER_FLAG_23
-#define SPECENUM_VALUE52 UTYF_USER_FLAG_24
-#define SPECENUM_VALUE53 UTYF_USER_FLAG_25
-#define SPECENUM_VALUE54 UTYF_USER_FLAG_26
-#define SPECENUM_VALUE55 UTYF_USER_FLAG_27
-#define SPECENUM_VALUE56 UTYF_USER_FLAG_28
-#define SPECENUM_VALUE57 UTYF_USER_FLAG_29
-#define SPECENUM_VALUE58 UTYF_USER_FLAG_30
-#define SPECENUM_VALUE59 UTYF_USER_FLAG_31
-#define SPECENUM_VALUE60 UTYF_USER_FLAG_32
-#define SPECENUM_VALUE61 UTYF_USER_FLAG_33
-#define SPECENUM_VALUE62 UTYF_USER_FLAG_34
-#define SPECENUM_VALUE63 UTYF_USER_FLAG_35
-#define SPECENUM_VALUE64 UTYF_USER_FLAG_36
-#define SPECENUM_VALUE65 UTYF_USER_FLAG_37
-#define SPECENUM_VALUE66 UTYF_USER_FLAG_38
-#define SPECENUM_VALUE67 UTYF_USER_FLAG_39
-#define SPECENUM_VALUE68 UTYF_USER_FLAG_40
-#define SPECENUM_VALUE69 UTYF_USER_FLAG_41
-#define SPECENUM_VALUE70 UTYF_USER_FLAG_42
-#define SPECENUM_VALUE71 UTYF_USER_FLAG_43
-#define SPECENUM_VALUE72 UTYF_USER_FLAG_44
-#define SPECENUM_VALUE73 UTYF_USER_FLAG_45
+#define SPECENUM_VALUE30 UTYF_USER_FLAG_1
+#define SPECENUM_VALUE31 UTYF_USER_FLAG_2
+#define SPECENUM_VALUE32 UTYF_USER_FLAG_3
+#define SPECENUM_VALUE33 UTYF_USER_FLAG_4
+#define SPECENUM_VALUE34 UTYF_USER_FLAG_5
+#define SPECENUM_VALUE35 UTYF_USER_FLAG_6
+#define SPECENUM_VALUE36 UTYF_USER_FLAG_7
+#define SPECENUM_VALUE37 UTYF_USER_FLAG_8
+#define SPECENUM_VALUE38 UTYF_USER_FLAG_9
+#define SPECENUM_VALUE39 UTYF_USER_FLAG_10
+#define SPECENUM_VALUE40 UTYF_USER_FLAG_11
+#define SPECENUM_VALUE41 UTYF_USER_FLAG_12
+#define SPECENUM_VALUE42 UTYF_USER_FLAG_13
+#define SPECENUM_VALUE43 UTYF_USER_FLAG_14
+#define SPECENUM_VALUE44 UTYF_USER_FLAG_15
+#define SPECENUM_VALUE45 UTYF_USER_FLAG_16
+#define SPECENUM_VALUE46 UTYF_USER_FLAG_17
+#define SPECENUM_VALUE47 UTYF_USER_FLAG_18
+#define SPECENUM_VALUE48 UTYF_USER_FLAG_19
+#define SPECENUM_VALUE49 UTYF_USER_FLAG_20
+#define SPECENUM_VALUE50 UTYF_USER_FLAG_21
+#define SPECENUM_VALUE51 UTYF_USER_FLAG_22
+#define SPECENUM_VALUE52 UTYF_USER_FLAG_23
+#define SPECENUM_VALUE53 UTYF_USER_FLAG_24
+#define SPECENUM_VALUE54 UTYF_USER_FLAG_25
+#define SPECENUM_VALUE55 UTYF_USER_FLAG_26
+#define SPECENUM_VALUE56 UTYF_USER_FLAG_27
+#define SPECENUM_VALUE57 UTYF_USER_FLAG_28
+#define SPECENUM_VALUE58 UTYF_USER_FLAG_29
+#define SPECENUM_VALUE59 UTYF_USER_FLAG_30
+#define SPECENUM_VALUE60 UTYF_USER_FLAG_31
+#define SPECENUM_VALUE61 UTYF_USER_FLAG_32
+#define SPECENUM_VALUE62 UTYF_USER_FLAG_33
+#define SPECENUM_VALUE63 UTYF_USER_FLAG_34
+#define SPECENUM_VALUE64 UTYF_USER_FLAG_35
+#define SPECENUM_VALUE65 UTYF_USER_FLAG_36
+#define SPECENUM_VALUE66 UTYF_USER_FLAG_37
+#define SPECENUM_VALUE67 UTYF_USER_FLAG_38
+#define SPECENUM_VALUE68 UTYF_USER_FLAG_39
+#define SPECENUM_VALUE69 UTYF_USER_FLAG_40
+#define SPECENUM_VALUE70 UTYF_USER_FLAG_41
+#define SPECENUM_VALUE71 UTYF_USER_FLAG_42
+#define SPECENUM_VALUE72 UTYF_USER_FLAG_43
+#define SPECENUM_VALUE73 UTYF_USER_FLAG_44
+#define SPECENUM_VALUE74 UTYF_USER_FLAG_45
+#define SPECENUM_VALUE75 UTYF_USER_FLAG_46
+#define SPECENUM_VALUE76 UTYF_USER_FLAG_47
+#define SPECENUM_VALUE77 UTYF_USER_FLAG_48
+#define SPECENUM_VALUE78 UTYF_USER_FLAG_49
+#define SPECENUM_VALUE79 UTYF_USER_FLAG_50
 /* Note that first role must have value next to last flag */
 
-#define UTYF_LAST_USER_FLAG UTYF_USER_FLAG_45
+#define UTYF_LAST_USER_FLAG UTYF_USER_FLAG_50
 #define MAX_NUM_USER_UNIT_FLAGS (UTYF_LAST_USER_FLAG - UTYF_USER_FLAG_1 + 1)
 #define SPECENUM_NAMEOVERRIDE
 #define SPECENUM_BITVECTOR bv_unit_type_flags
@@ -327,96 +343,99 @@ struct unit_class {
    a role for "settlers", because we can just use UTYF_SETTLERS.
    (Now have to consider ACTION_FOUND_CITY too)
    So we make sure flag values and role values are distinct,
-   so some functions can use them interchangably.
+   so some functions can use them interchangeably.
    See data/classic/units.ruleset for documentation of their effects.
 */
 #define L_FIRST (UTYF_LAST_USER_FLAG + 1)
 
 #define SPECENUM_NAME unit_role_id
-/* is built first when city established */
-#define SPECENUM_VALUE74 L_FIRSTBUILD
-#define SPECENUM_VALUE74NAME N_("?unitflag:FirstBuild")
-/* initial explorer unit */
-#define SPECENUM_VALUE75 L_EXPLORER
-#define SPECENUM_VALUE75NAME N_("?unitflag:Explorer")
-/* can be found in hut */
-#define SPECENUM_VALUE76 L_HUT
-#define SPECENUM_VALUE76NAME N_("?unitflag:Hut")
-/* can be found in hut, tech required */
-#define SPECENUM_VALUE77 L_HUT_TECH
-#define SPECENUM_VALUE77NAME N_("?unitflag:HutTech")
-/* is created in Partisan circumstances */
-#define SPECENUM_VALUE78 L_PARTISAN
-#define SPECENUM_VALUE78NAME N_("?unitflag:Partisan")
-/* ok on defense (AI) */
-#define SPECENUM_VALUE79 L_DEFEND_OK
-#define SPECENUM_VALUE79NAME N_("?unitflag:DefendOk")
-/* primary purpose is defense (AI) */
-#define SPECENUM_VALUE80 L_DEFEND_GOOD
-#define SPECENUM_VALUE80NAME N_("?unitflag:DefendGood")
-/* is useful for ferrying (AI) */
-#define SPECENUM_VALUE81 L_FERRYBOAT
-#define SPECENUM_VALUE81NAME N_("?unitflag:FerryBoat")
-/* barbarians unit, land only */
-#define SPECENUM_VALUE82 L_BARBARIAN
-#define SPECENUM_VALUE82NAME N_("?unitflag:Barbarian")
-/* barbarians unit, global tech required */
-#define SPECENUM_VALUE83 L_BARBARIAN_TECH
-#define SPECENUM_VALUE83NAME N_("?unitflag:BarbarianTech")
-/* barbarian boat */
-#define SPECENUM_VALUE84 L_BARBARIAN_BOAT
-#define SPECENUM_VALUE84NAME N_("?unitflag:BarbarianBoat")
-/* what barbarians should build */
-#define SPECENUM_VALUE85 L_BARBARIAN_BUILD
-#define SPECENUM_VALUE85NAME N_("?unitflag:BarbarianBuild")
-/* barbarians build when global tech */
-#define SPECENUM_VALUE86 L_BARBARIAN_BUILD_TECH
-#define SPECENUM_VALUE86NAME N_("?unitflag:BarbarianBuildTech")
-/* barbarian leader */
-#define SPECENUM_VALUE87 L_BARBARIAN_LEADER
-#define SPECENUM_VALUE87NAME N_("?unitflag:BarbarianLeader")
-/* sea raider unit */
-#define SPECENUM_VALUE88 L_BARBARIAN_SEA
-#define SPECENUM_VALUE88NAME N_("?unitflag:BarbarianSea")
-/* sea raider unit, global tech required */
-#define SPECENUM_VALUE89 L_BARBARIAN_SEA_TECH
-#define SPECENUM_VALUE89NAME N_("?unitflag:BarbarianSeaTech")
+/* Built first when city established */
+#define SPECENUM_VALUE80 L_FIRSTBUILD
+#define SPECENUM_VALUE80NAME N_("?unitflag:FirstBuild")
+/* Explorer unit */
+#define SPECENUM_VALUE81 L_EXPLORER
+#define SPECENUM_VALUE81NAME N_("?unitflag:Explorer")
+/* Can be found in hut */
+#define SPECENUM_VALUE82 L_HUT
+#define SPECENUM_VALUE82NAME N_("?unitflag:Hut")
+/* Can be found in hut, tech required */
+#define SPECENUM_VALUE83 L_HUT_TECH
+#define SPECENUM_VALUE83NAME N_("?unitflag:HutTech")
+/* Created in Partisan circumstances */
+#define SPECENUM_VALUE84 L_PARTISAN
+#define SPECENUM_VALUE84NAME N_("?unitflag:Partisan")
+/* Ok on defense (AI) */
+#define SPECENUM_VALUE85 L_DEFEND_OK
+#define SPECENUM_VALUE85NAME N_("?unitflag:DefendOk")
+/* Primary purpose is defense (AI) */
+#define SPECENUM_VALUE86 L_DEFEND_GOOD
+#define SPECENUM_VALUE86NAME N_("?unitflag:DefendGood")
+/* Useful for ferrying (AI) */
+#define SPECENUM_VALUE87 L_FERRYBOAT
+#define SPECENUM_VALUE87NAME N_("?unitflag:FerryBoat")
+/* Barbarians unit, land only */
+#define SPECENUM_VALUE88 L_BARBARIAN
+#define SPECENUM_VALUE88NAME N_("?unitflag:Barbarian")
+/* Barbarians unit, global tech required */
+#define SPECENUM_VALUE89 L_BARBARIAN_TECH
+#define SPECENUM_VALUE89NAME N_("?unitflag:BarbarianTech")
+/* Barbarian boat */
+#define SPECENUM_VALUE90 L_BARBARIAN_BOAT
+#define SPECENUM_VALUE90NAME N_("?unitflag:BarbarianBoat")
+/* Barbarians boat, global tech required */
+#define SPECENUM_VALUE91 L_BARBARIAN_BOAT_TECH
+#define SPECENUM_VALUE91NAME N_("?unitflag:BarbarianBoatTech")
+/* What barbarians should build */
+#define SPECENUM_VALUE92 L_BARBARIAN_BUILD
+#define SPECENUM_VALUE92NAME N_("?unitflag:BarbarianBuild")
+/* Barbarians build when global tech */
+#define SPECENUM_VALUE93 L_BARBARIAN_BUILD_TECH
+#define SPECENUM_VALUE93NAME N_("?unitflag:BarbarianBuildTech")
+/* Barbarian leader */
+#define SPECENUM_VALUE94 L_BARBARIAN_LEADER
+#define SPECENUM_VALUE94NAME N_("?unitflag:BarbarianLeader")
+/* Sea raider unit */
+#define SPECENUM_VALUE95 L_BARBARIAN_SEA
+#define SPECENUM_VALUE95NAME N_("?unitflag:BarbarianSea")
+/* Sea raider unit, global tech required */
+#define SPECENUM_VALUE96 L_BARBARIAN_SEA_TECH
+#define SPECENUM_VALUE96NAME N_("?unitflag:BarbarianSeaTech")
 /* StartUnit: Cities */
-#define SPECENUM_VALUE90 L_START_CITIES
-#define SPECENUM_VALUE90NAME N_("?unitflag:CitiesStartUnit")
+#define SPECENUM_VALUE97 L_START_CITIES
+#define SPECENUM_VALUE97NAME N_("?unitflag:CitiesStartUnit")
 /* StartUnit: Worker */
-#define SPECENUM_VALUE91 L_START_WORKER
-#define SPECENUM_VALUE91NAME N_("?unitflag:WorkerStartUnit")
+#define SPECENUM_VALUE98 L_START_WORKER
+#define SPECENUM_VALUE98NAME N_("?unitflag:WorkerStartUnit")
 /* StartUnit: Explorer */
-#define SPECENUM_VALUE92 L_START_EXPLORER
-#define SPECENUM_VALUE92NAME N_("?unitflag:ExplorerStartUnit")
+#define SPECENUM_VALUE99 L_START_EXPLORER
+#define SPECENUM_VALUE99NAME N_("?unitflag:ExplorerStartUnit")
 /* StartUnit: King */
-#define SPECENUM_VALUE93 L_START_KING
-#define SPECENUM_VALUE93NAME N_("?unitflag:KingStartUnit")
+#define SPECENUM_VALUE100 L_START_KING
+#define SPECENUM_VALUE100NAME N_("?unitflag:KingStartUnit")
 /* StartUnit: Diplomat */
-#define SPECENUM_VALUE94 L_START_DIPLOMAT
-#define SPECENUM_VALUE94NAME N_("?unitflag:DiplomatStartUnit")
+#define SPECENUM_VALUE101 L_START_DIPLOMAT
+#define SPECENUM_VALUE101NAME N_("?unitflag:DiplomatStartUnit")
 /* StartUnit: Ferryboat */
-#define SPECENUM_VALUE95 L_START_FERRY
-#define SPECENUM_VALUE95NAME N_("?unitflag:FerryStartUnit")
+#define SPECENUM_VALUE102 L_START_FERRY
+#define SPECENUM_VALUE102NAME N_("?unitflag:FerryStartUnit")
 /* StartUnit: DefendOk */
-#define SPECENUM_VALUE96 L_START_DEFEND_OK
-#define SPECENUM_VALUE96NAME N_("?unitflag:DefendOkStartUnit")
+#define SPECENUM_VALUE103 L_START_DEFEND_OK
+#define SPECENUM_VALUE103NAME N_("?unitflag:DefendOkStartUnit")
 /* StartUnit: DefendGood */
-#define SPECENUM_VALUE97 L_START_DEFEND_GOOD
-#define SPECENUM_VALUE97NAME N_("?unitflag:DefendGoodStartUnit")
+#define SPECENUM_VALUE104 L_START_DEFEND_GOOD
+#define SPECENUM_VALUE104NAME N_("?unitflag:DefendGoodStartUnit")
 /* StartUnit: AttackFast */
-#define SPECENUM_VALUE98 L_START_ATTACK_FAST
-#define SPECENUM_VALUE98NAME N_("?unitflag:AttackFastStartUnit")
+#define SPECENUM_VALUE105 L_START_ATTACK_FAST
+#define SPECENUM_VALUE105NAME N_("?unitflag:AttackFastStartUnit")
 /* StartUnit: AttackStrong */
-#define SPECENUM_VALUE99 L_START_ATTACK_STRONG
-#define SPECENUM_VALUE99NAME N_("?unitflag:AttackStrongStartUnit")
+#define SPECENUM_VALUE106 L_START_ATTACK_STRONG
+#define SPECENUM_VALUE106NAME N_("?unitflag:AttackStrongStartUnit")
 /* AI hunter type unit */
-#define SPECENUM_VALUE100 L_HUNTER
-#define SPECENUM_VALUE100NAME N_("?unitflag:Hunter")
-/* can improve terrain */
-#define SPECENUM_VALUE101 L_SETTLERS
-#define SPECENUM_VALUE101NAME N_("?unitflag:Settlers")
+#define SPECENUM_VALUE107 L_HUNTER
+#define SPECENUM_VALUE107NAME N_("?unitflag:Hunter")
+/* Can improve terrain */
+#define SPECENUM_VALUE108 L_SETTLERS
+#define SPECENUM_VALUE108NAME N_("?unitflag:Settlers")
 #define L_LAST (L_SETTLERS + 1)
 
 #include "specenum_gen.h"
@@ -485,6 +504,7 @@ struct unit_type {
   void *ruledit_dlg;
   char graphic_str[MAX_LEN_NAME];
   char graphic_alt[MAX_LEN_NAME];
+  char graphic_alt2[MAX_LEN_NAME];
   char sound_move[MAX_LEN_NAME];
   char sound_move_alt[MAX_LEN_NAME];
   char sound_fight[MAX_LEN_NAME];
@@ -497,7 +517,6 @@ struct unit_type {
   int move_rate;
   int unknown_move_cost;                /* See utype_unknown_move_cost(). */
 
-  struct advance *require_advance;      /* May be NULL */
   struct requirement_vector build_reqs;
 
   int vision_radius_sq;
@@ -566,12 +585,6 @@ struct unit_type {
                                  * bonus multiplier), for other 0 */
   } cache;
 
-  /* Used to upgrade the ruleset format version. */
-  struct {
-    int paratroopers_mr_req;
-    int paratroopers_mr_sub;
-  } rscompat_cache;
-
   void *ais[FREECIV_AI_MOD_LAST];
 };
 
@@ -816,7 +829,8 @@ bool utype_player_already_has_this_unique(const struct player *pplayer,
                                           const struct unit_type *putype);
 
 bool can_player_build_unit_direct(const struct player *p,
-                                  const struct unit_type *punittype);
+                                  const struct unit_type *punittype,
+                                  bool consider_reg_impr_req);
 bool can_player_build_unit_later(const struct player *p,
                                  const struct unit_type *punittype);
 bool can_player_build_unit_now(const struct player *p,
@@ -858,6 +872,21 @@ const struct unit_type *unit_type_array_last(void);
     }                                                                   \
   } unit_type_iterate_end;
 
+#define unit_tech_reqs_iterate(_utype_, _p)                             \
+do {                                                                    \
+ requirement_vector_iterate(&(_utype_)->build_reqs, preq_##_p) {        \
+ if (preq_##_p->source.kind == VUT_ADVANCE) {                           \
+   struct advance *_p = preq_##_p->source.value.advance;
+
+#define unit_tech_reqs_iterate_end                                      \
+    }                                                                   \
+  } requirement_vector_iterate_end;                                     \
+} while (FALSE);
+
+/* Used on client to show just one req */
+struct advance *utype_primary_tech_req(const struct unit_type *ptype);
+bool is_tech_req_for_utype(const struct unit_type *ptype,
+                           struct advance *padv);
 
 void *utype_ai_data(const struct unit_type *ptype, const struct ai_type *ai);
 void utype_set_ai_data(struct unit_type *ptype, const struct ai_type *ai,
