@@ -224,6 +224,7 @@ static void count_my_units(struct player *pplayer)
         || unit_can_do_action(punit, ACTION_PARADROP_FRIGHTEN_CONQUER)
         || unit_can_do_action(punit, ACTION_PARADROP_ENTER)
         || unit_can_do_action(punit, ACTION_PARADROP_ENTER_CONQUER)) {
+      /* TODO: Cover also teleporting */
       adv->stats.units.paratroopers++;
     }
     if (utype_can_do_action(punit->utype, ACTION_AIRLIFT)) {
@@ -269,10 +270,10 @@ bool adv_data_phase_init(struct player *pplayer, bool is_new_phase)
     int i = 0;
 
     /* Conventional nukes */
-    action_list_add_all_by_result(nuke_actions, &i, ACTRES_NUKE);
-    action_list_add_all_by_result(nuke_actions, &i, ACTRES_NUKE_UNITS);
+    action_array_add_all_by_result(nuke_actions, &i, ACTRES_NUKE);
+    action_array_add_all_by_result(nuke_actions, &i, ACTRES_NUKE_UNITS);
     /* TODO: worry about spy nuking too? */
-    action_list_end(nuke_actions, i);
+    action_array_end(nuke_actions, i);
   }
 
   fc_assert_ret_val(adv != NULL, FALSE);
@@ -290,12 +291,22 @@ bool adv_data_phase_init(struct player *pplayer, bool is_new_phase)
 
   adv->num_continents    = wld.map.num_continents;
   adv->num_oceans        = wld.map.num_oceans;
-  adv->threats.continent = fc_calloc(adv->num_continents + 1, sizeof(bool));
+  adv->continents        = fc_calloc(adv->num_continents + 1, sizeof(struct adv_area_info));
+  adv->oceans            = fc_calloc(adv->num_oceans + 1, sizeof(struct adv_area_info));
   adv->threats.invasions = FALSE;
-  adv->threats.nuclear   = 0; /* none */
-  adv->threats.ocean     = fc_calloc(adv->num_oceans + 1, sizeof(bool));
+  adv->threats.nuclear   = 0; /* None */
   adv->threats.igwall    = FALSE;
 
+  whole_map_iterate(&(wld.map), ptile) {
+    Continent_id cont = tile_continent(ptile);
+
+    if (cont >= 0) {
+      adv->continents[cont].size++;
+    } else {
+      adv->oceans[-cont].size++;
+    }
+  } whole_map_iterate_end;
+
   players_iterate(aplayer) {
     if (!adv_is_player_dangerous(pplayer, aplayer)) {
       continue;
@@ -307,8 +318,9 @@ bool adv_data_phase_init(struct player *pplayer, bool is_new_phase)
      * coastal fortresses and hunting down enemy transports. */
     city_list_iterate(aplayer->cities, acity) {
       Continent_id continent = tile_continent(acity->tile);
+
       if (continent >= 0) {
-        adv->threats.continent[continent] = TRUE;
+        adv->continents[continent].threat = TRUE;
       }
     } city_list_iterate_end;
 
@@ -343,13 +355,13 @@ bool adv_data_phase_init(struct player *pplayer, bool is_new_phase)
           if (is_ocean_tile(unit_tile(punit))) {
             Continent_id continent = tile_continent(unit_tile(punit));
 
-            adv->threats.ocean[-continent] = TRUE;
+            adv->oceans[-continent].threat = TRUE;
           } else {
             adjc_iterate(&(wld.map), unit_tile(punit), tile2) {
               if (is_ocean_tile(tile2)) {
                 Continent_id continent = tile_continent(tile2);
 
-                adv->threats.ocean[-continent] = TRUE;
+                adv->oceans[-continent].threat = TRUE;
               }
             } adjc_iterate_end;
           }
@@ -380,7 +392,7 @@ bool adv_data_phase_init(struct player *pplayer, bool is_new_phase)
         struct unit_type *nuke =
             get_role_unit(action_id_get_role(act_id), i);
 
-        if (can_player_build_unit_direct(aplayer, nuke)) {
+        if (can_player_build_unit_direct(aplayer, nuke, FALSE)) {
           adv->threats.nuclear = 1;
         }
       }
@@ -405,7 +417,7 @@ bool adv_data_phase_init(struct player *pplayer, bool is_new_phase)
     if (is_ocean_tile(ptile)) {
       if (adv->explore.sea_done && has_handicap(pplayer, H_TARGETS) 
           && !map_is_known(ptile, pplayer)) {
-	/* We're not done there. */
+        /* We're not done there. */
         adv->explore.sea_done = FALSE;
         adv->explore.ocean[-continent] = TRUE;
       }
@@ -566,11 +578,11 @@ void adv_data_phase_done(struct player *pplayer)
   free(adv->explore.continent);
   adv->explore.continent = NULL;
 
-  free(adv->threats.continent);
-  adv->threats.continent = NULL;
+  free(adv->continents);
+  adv->continents = NULL;
 
-  free(adv->threats.ocean);
-  adv->threats.ocean = NULL;
+  free(adv->oceans);
+  adv->oceans = NULL;
 
   free(adv->stats.cities);
   adv->stats.cities = NULL;
@@ -827,6 +839,7 @@ adv_want adv_gov_action_immunity_want(struct government *gov)
     case ACTRES_ATTACK:
     case ACTRES_SPY_INCITE_CITY:
     case ACTRES_CONQUER_CITY:
+    case ACTRES_COLLECT_RANSOM:
       bonus += 4;
       break;
     case ACTRES_SPY_BRIBE_UNIT:
@@ -841,6 +854,7 @@ adv_want adv_gov_action_immunity_want(struct government *gov)
       break;
     case ACTRES_CONQUER_EXTRAS:
     case ACTRES_PILLAGE:
+    case ACTRES_WIPE_UNITS:
       bonus += 0.2;
       break;
     case ACTRES_HUT_ENTER:
@@ -875,12 +889,16 @@ adv_want adv_gov_action_immunity_want(struct government *gov)
       break;
 
     case ACTRES_UNIT_MOVE:
+    case ACTRES_TELEPORT:
+    case ACTRES_TELEPORT_CONQUER:
+    case ACTRES_ENABLER_CHECK:
     case ACTRES_MARKETPLACE:
     case ACTRES_FOUND_CITY:
     case ACTRES_DISBAND_UNIT:
     case ACTRES_PARADROP:
     case ACTRES_PARADROP_CONQUER:
     case ACTRES_FORTIFY:
+    case ACTRES_SPY_ESCAPE:
       /* Wants the ability to do this to it self. Don't want others
        * to target it. Do nothing since action_immune_government()
        * doesn't separate based on who the actor is. */
@@ -905,18 +923,20 @@ adv_want adv_gov_action_immunity_want(struct government *gov)
     case ACTRES_BASE:
     case ACTRES_MINE:
     case ACTRES_IRRIGATE:
-    case ACTRES_CLEAN_POLLUTION:
-    case ACTRES_CLEAN_FALLOUT:
-    case ACTRES_TRANSPORT_ALIGHT:
+    case ACTRES_CLEAN:
+    case ACTRES_TRANSPORT_DEBOARD:
     case ACTRES_TRANSPORT_UNLOAD:
     case ACTRES_TRANSPORT_DISEMBARK:
     case ACTRES_TRANSPORT_BOARD:
+    case ACTRES_TRANSPORT_LOAD:
     case ACTRES_TRANSPORT_EMBARK:
       /* Could be good. An embassy gives permanent contact. A trade
-       * route gives gold per turn. Join city gives population. Help
-       * wonder gives shields. */
+       * route gives gold per turn. Join city gives population.
+       * Help wonder gives shields. */
       /* TODO: Individual and well balanced values. */
       break;
+
+    ASSERT_UNUSED_ACTRES_CASES;
     }
   } action_iterate_end;
 
