@@ -32,6 +32,7 @@
 #include "government.h"
 #include "map.h"
 #include "movement.h"
+#include "nation.h"
 #include "packets.h"
 #include "unitlist.h"
 
@@ -96,32 +97,30 @@ void auto_settlers_ruleset_init(void)
   int i;
 
   i = 0;
-  action_list_add_all_by_result(as_actions_transform, &i,
-                                ACTRES_CULTIVATE);
-  action_list_add_all_by_result(as_actions_transform, &i,
-                                ACTRES_PLANT);
-  action_list_add_all_by_result(as_actions_transform, &i,
-                                ACTRES_TRANSFORM_TERRAIN);
-  action_list_end(as_actions_transform, i);
+  action_array_add_all_by_result(as_actions_transform, &i,
+                                 ACTRES_CULTIVATE);
+  action_array_add_all_by_result(as_actions_transform, &i,
+                                 ACTRES_PLANT);
+  action_array_add_all_by_result(as_actions_transform, &i,
+                                 ACTRES_TRANSFORM_TERRAIN);
+  action_array_end(as_actions_transform, i);
 
   i = 0;
-  action_list_add_all_by_result(as_actions_extra, &i,
-                                ACTRES_IRRIGATE);
-  action_list_add_all_by_result(as_actions_extra, &i,
-                                ACTRES_MINE);
-  action_list_add_all_by_result(as_actions_extra, &i,
-                                ACTRES_ROAD);
-  action_list_add_all_by_result(as_actions_extra, &i,
-                                ACTRES_BASE);
-  action_list_end(as_actions_extra, i);
+  action_array_add_all_by_result(as_actions_extra, &i,
+                                 ACTRES_IRRIGATE);
+  action_array_add_all_by_result(as_actions_extra, &i,
+                                 ACTRES_MINE);
+  action_array_add_all_by_result(as_actions_extra, &i,
+                                 ACTRES_ROAD);
+  action_array_add_all_by_result(as_actions_extra, &i,
+                                 ACTRES_BASE);
+  action_array_end(as_actions_extra, i);
 
   i = 0;
-  action_list_add_all_by_result(as_actions_rmextra, &i,
-                                ACTRES_CLEAN_POLLUTION);
-  action_list_add_all_by_result(as_actions_rmextra, &i,
-                                ACTRES_CLEAN_FALLOUT);
+  action_array_add_all_by_result(as_actions_rmextra, &i,
+                                 ACTRES_CLEAN);
   /* We could have ACTRES_PILLAGE here, but currently we don't */
-  action_list_end(as_actions_rmextra, i);
+  action_array_end(as_actions_rmextra, i);
 }
 
 /**********************************************************************//**
@@ -601,8 +600,8 @@ adv_want settler_evaluate_improvements(const struct civ_map *nmap,
                                                       ptile,
                                                       parameter.omniscience,
                                                       ptile, pextra))) {
-                    act = actres_get_activity(taction->result);
-                    eval_act = actres_get_activity(taction->result);
+                    act = action_get_activity(taction);
+                    eval_act = action_get_activity(taction);
                     break;
                   }
                 }
@@ -620,7 +619,7 @@ adv_want settler_evaluate_improvements(const struct civ_map *nmap,
                                                       ptile,
                                                       parameter.omniscience,
                                                       ptile, pextra))) {
-                    act = actres_get_activity(taction->result);
+                    act = action_get_activity(taction);
                     break;
                   }
                 }
@@ -1146,7 +1145,7 @@ bool adv_settler_safe_tile(const struct civ_map *nmap,
                            struct tile *ptile)
 {
   unit_list_iterate(ptile->units, defender) {
-    if (is_military_unit(defender)) {
+    if (is_guard_unit(defender)) {
       return TRUE;
     }
   } unit_list_iterate_end;
@@ -1170,7 +1169,8 @@ void auto_settlers_player(struct player *pplayer)
 
   state = fc_calloc(MAP_INDEX_SIZE, sizeof(*state));
 
-  as_timer = timer_renew(as_timer, TIMER_CPU, TIMER_DEBUG);
+  as_timer = timer_renew(as_timer, TIMER_CPU, TIMER_DEBUG,
+                         as_timer != NULL ? NULL : "autosettlers");
   timer_start(as_timer);
 
   if (is_ai(pplayer)) {
@@ -1309,7 +1309,7 @@ bool auto_settlers_speculate_can_act_at(const struct unit *punit,
                         FALSE);
       break;
     case ATK_UNITS:
-      return action_prob_possible(action_speculate_unit_on_units(
+      return action_prob_possible(action_speculate_unit_on_stack(
                                     nmap, paction->id,
                                     punit, unit_home(punit), ptile,
                                     omniscient_cheat,
