@@ -1,12 +1,9 @@
-
- /****************************************************************************
- *                       THIS FILE WAS GENERATED                             *
- * Script: common/generate_packets.py                                        *
- * Input:  common/networking/packets.def                                     *
- *                       DO NOT CHANGE THIS FILE                             *
- ****************************************************************************/
-
-
+ /**************************************************************************
+ *                         THIS FILE WAS GENERATED                         *
+ * Script: common/generate_packets.py                                      *
+ * Input:  common/networking/packets.def                                   *
+ *                         DO NOT CHANGE THIS FILE                         *
+ **************************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #include <fc_config.h>
@@ -43,78 +40,68 @@ bool server_handle_packet(enum packet_type type, const void *packet,
 
   case PACKET_CITY_SELL:
     handle_city_sell(pplayer,
-      ((const struct packet_city_sell *)packet)->city_id16,
-      ((const struct packet_city_sell *)packet)->city_id32,
+      ((const struct packet_city_sell *)packet)->city_id,
       ((const struct packet_city_sell *)packet)->build_id);
     return TRUE;
 
   case PACKET_CITY_BUY:
     handle_city_buy(pplayer,
-      ((const struct packet_city_buy *)packet)->city_id16,
-      ((const struct packet_city_buy *)packet)->city_id32);
+      ((const struct packet_city_buy *)packet)->city_id);
     return TRUE;
 
   case PACKET_CITY_CHANGE:
     handle_city_change(pplayer,
-      ((const struct packet_city_change *)packet)->city_id16,
-      ((const struct packet_city_change *)packet)->city_id32,
+      ((const struct packet_city_change *)packet)->city_id,
       ((const struct packet_city_change *)packet)->production_kind,
       ((const struct packet_city_change *)packet)->production_value);
     return TRUE;
 
   case PACKET_CITY_WORKLIST:
     handle_city_worklist(pplayer,
-      ((const struct packet_city_worklist *)packet)->city_id16,
-      ((const struct packet_city_worklist *)packet)->city_id32,
+      ((const struct packet_city_worklist *)packet)->city_id,
       &((const struct packet_city_worklist *)packet)->worklist);
     return TRUE;
 
   case PACKET_CITY_MAKE_SPECIALIST:
     handle_city_make_specialist(pplayer,
-      ((const struct packet_city_make_specialist *)packet)->city_id16,
-      ((const struct packet_city_make_specialist *)packet)->city_id32,
+      ((const struct packet_city_make_specialist *)packet)->city_id,
       ((const struct packet_city_make_specialist *)packet)->tile_id);
     return TRUE;
 
   case PACKET_CITY_MAKE_WORKER:
     handle_city_make_worker(pplayer,
-      ((const struct packet_city_make_worker *)packet)->city_id16,
-      ((const struct packet_city_make_worker *)packet)->city_id32,
+      ((const struct packet_city_make_worker *)packet)->city_id,
       ((const struct packet_city_make_worker *)packet)->tile_id);
     return TRUE;
 
   case PACKET_CITY_CHANGE_SPECIALIST:
     handle_city_change_specialist(pplayer,
-      ((const struct packet_city_change_specialist *)packet)->city_id16,
-      ((const struct packet_city_change_specialist *)packet)->city_id32,
+      ((const struct packet_city_change_specialist *)packet)->city_id,
       ((const struct packet_city_change_specialist *)packet)->from,
       ((const struct packet_city_change_specialist *)packet)->to);
     return TRUE;
 
   case PACKET_CITY_RENAME:
     handle_city_rename(pplayer,
-      ((const struct packet_city_rename *)packet)->city_id16,
-      ((const struct packet_city_rename *)packet)->city_id32,
+      ((const struct packet_city_rename *)packet)->city_id,
       ((const struct packet_city_rename *)packet)->name);
     return TRUE;
 
   case PACKET_CITY_OPTIONS_REQ:
     handle_city_options_req(pplayer,
-      ((const struct packet_city_options_req *)packet)->city_id16,
-      ((const struct packet_city_options_req *)packet)->city_id32,
-      ((const struct packet_city_options_req *)packet)->options);
+      ((const struct packet_city_options_req *)packet)->city_id,
+      ((const struct packet_city_options_req *)packet)->options,
+      ((const struct packet_city_options_req *)packet)->wl_cb);
     return TRUE;
 
   case PACKET_CITY_REFRESH:
     handle_city_refresh(pplayer,
-      ((const struct packet_city_refresh *)packet)->city_id16,
-      ((const struct packet_city_refresh *)packet)->city_id32);
+      ((const struct packet_city_refresh *)packet)->city_id);
     return TRUE;
 
   case PACKET_CITY_NAME_SUGGESTION_REQ:
     handle_city_name_suggestion_req(pplayer,
-      ((const struct packet_city_name_suggestion_req *)packet)->unit_id16,
-      ((const struct packet_city_name_suggestion_req *)packet)->unit_id32);
+      ((const struct packet_city_name_suggestion_req *)packet)->unit_id);
     return TRUE;
 
   case PACKET_CITY_RALLY_POINT:
@@ -174,8 +161,7 @@ bool server_handle_packet(enum packet_type type, const void *packet,
 
   case PACKET_UNIT_SSCS_SET:
     handle_unit_sscs_set(pplayer,
-      ((const struct packet_unit_sscs_set *)packet)->unit_id16,
-      ((const struct packet_unit_sscs_set *)packet)->unit_id32,
+      ((const struct packet_unit_sscs_set *)packet)->unit_id,
       ((const struct packet_unit_sscs_set *)packet)->type,
       ((const struct packet_unit_sscs_set *)packet)->value);
     return TRUE;
@@ -186,15 +172,13 @@ bool server_handle_packet(enum packet_type type, const void *packet,
 
   case PACKET_UNIT_SERVER_SIDE_AGENT_SET:
     handle_unit_server_side_agent_set(pplayer,
-      ((const struct packet_unit_server_side_agent_set *)packet)->unit_id16,
-      ((const struct packet_unit_server_side_agent_set *)packet)->unit_id32,
+      ((const struct packet_unit_server_side_agent_set *)packet)->unit_id,
       ((const struct packet_unit_server_side_agent_set *)packet)->agent);
     return TRUE;
 
   case PACKET_UNIT_ACTION_QUERY:
     handle_unit_action_query(pconn,
-      ((const struct packet_unit_action_query *)packet)->actor_id16,
-      ((const struct packet_unit_action_query *)packet)->actor_id32,
+      ((const struct packet_unit_action_query *)packet)->actor_id,
       ((const struct packet_unit_action_query *)packet)->target_id,
       ((const struct packet_unit_action_query *)packet)->action_type,
       ((const struct packet_unit_action_query *)packet)->request_kind);
@@ -206,17 +190,26 @@ bool server_handle_packet(enum packet_type type, const void *packet,
     return TRUE;
 
   case PACKET_UNIT_DO_ACTION:
-    handle_unit_do_action(pplayer, packet);
+    handle_unit_do_action(pplayer,
+      ((const struct packet_unit_do_action *)packet)->actor_id,
+      ((const struct packet_unit_do_action *)packet)->target_id,
+      ((const struct packet_unit_do_action *)packet)->sub_tgt_id,
+      ((const struct packet_unit_do_action *)packet)->name,
+      ((const struct packet_unit_do_action *)packet)->action_type);
     return TRUE;
 
   case PACKET_UNIT_GET_ACTIONS:
-    handle_unit_get_actions(pconn, packet);
+    handle_unit_get_actions(pconn,
+      ((const struct packet_unit_get_actions *)packet)->actor_unit_id,
+      ((const struct packet_unit_get_actions *)packet)->target_unit_id,
+      ((const struct packet_unit_get_actions *)packet)->target_tile_id,
+      ((const struct packet_unit_get_actions *)packet)->target_extra_id,
+      ((const struct packet_unit_get_actions *)packet)->request_kind);
     return TRUE;
 
   case PACKET_UNIT_CHANGE_ACTIVITY:
     handle_unit_change_activity(pplayer,
-      ((const struct packet_unit_change_activity *)packet)->unit_id16,
-      ((const struct packet_unit_change_activity *)packet)->unit_id32,
+      ((const struct packet_unit_change_activity *)packet)->unit_id,
       ((const struct packet_unit_change_activity *)packet)->activity,
       ((const struct packet_unit_change_activity *)packet)->target);
     return TRUE;
@@ -292,6 +285,10 @@ bool server_handle_packet(enum packet_type type, const void *packet,
     handle_single_want_hack_req(pconn, packet);
     return TRUE;
 
+  case PACKET_RULESET_SELECT:
+    handle_ruleset_select(pconn, packet);
+    return TRUE;
+
   case PACKET_SAVE_SCENARIO:
     handle_save_scenario(pconn,
       ((const struct packet_save_scenario *)packet)->name);
@@ -368,8 +365,7 @@ bool server_handle_packet(enum packet_type type, const void *packet,
 
   case PACKET_EDIT_UNIT_REMOVE_BY_ID:
     handle_edit_unit_remove_by_id(pconn,
-      ((const struct packet_edit_unit_remove_by_id *)packet)->id16,
-      ((const struct packet_edit_unit_remove_by_id *)packet)->id32);
+      ((const struct packet_edit_unit_remove_by_id *)packet)->id);
     return TRUE;
 
   case PACKET_EDIT_UNIT:
@@ -386,8 +382,7 @@ bool server_handle_packet(enum packet_type type, const void *packet,
 
   case PACKET_EDIT_CITY_REMOVE:
     handle_edit_city_remove(pconn,
-      ((const struct packet_edit_city_remove *)packet)->id16,
-      ((const struct packet_edit_city_remove *)packet)->id32);
+      ((const struct packet_edit_city_remove *)packet)->id);
     return TRUE;
 
   case PACKET_EDIT_CITY:
@@ -425,6 +420,22 @@ bool server_handle_packet(enum packet_type type, const void *packet,
       ((const struct packet_edit_scenario_desc *)packet)->scenario_desc);
     return TRUE;
 
+  case PACKET_SYNC_SERIAL:
+    handle_sync_serial(pconn,
+      ((const struct packet_sync_serial *)packet)->serial);
+    return TRUE;
+
+  case PACKET_WEB_CMA_SET:
+    handle_web_cma_set(pplayer,
+      ((const struct packet_web_cma_set *)packet)->id,
+      &((const struct packet_web_cma_set *)packet)->cm_parameter);
+    return TRUE;
+
+  case PACKET_WEB_CMA_CLEAR:
+    handle_web_cma_clear(pplayer,
+      ((const struct packet_web_cma_clear *)packet)->id);
+    return TRUE;
+
   default:
     return FALSE;
   }
