@@ -83,6 +83,9 @@ static void map_change_own_seen(struct player *pplayer,
 static inline int map_get_seen(const struct player *pplayer,
                                const struct tile *ptile,
                                enum vision_layer vlayer);
+static inline bool map_is_also_seen(const struct tile *ptile,
+                                    const struct player *pplayer,
+                                    enum vision_layer vlayer);
 
 static bool is_claimable_ocean(struct tile *ptile, struct tile *source,
                                struct player *pplayer);
@@ -482,7 +485,7 @@ bool send_tile_suppression(bool now)
   the tile. If dest is NULL, sends to all clients (game.est_connections)
   which know and see tile.
 
-  Note that this function does not update the playermap.  For that call
+  Note that this function does not update the playermap. For that call
   update_tile_knowledge().
 **************************************************************************/
 void send_tile_info(struct conn_list *dest, struct tile *ptile,
@@ -514,12 +517,17 @@ void send_tile_info(struct conn_list *dest, struct tile *ptile,
 
   conn_list_iterate(dest, pconn) {
     struct player *pplayer = pconn->playing;
+    bool known;
 
     if (NULL == pplayer && !pconn->observer) {
       continue;
     }
 
-    if (!pplayer || map_is_known_and_seen(ptile, pplayer, V_MAIN)) {
+    if (pplayer != NULL) {
+      known = map_is_known(ptile, pplayer);
+    }
+
+    if (pplayer == NULL || (known && map_is_also_seen(ptile, pplayer, V_MAIN))) {
       struct extra_type *resource;
 
       info.known = TILE_KNOWN_SEEN;
@@ -528,10 +536,9 @@ void send_tile_info(struct conn_list *dest, struct tile *ptile,
       eowner = extra_owner(ptile);
       info.owner = (owner ? player_number(owner) : MAP_TILE_OWNER_NULL);
       info.extras_owner = (eowner ? player_number(eowner) : MAP_TILE_OWNER_NULL);
-      info.worked32 = (NULL != tile_worked(ptile))
-                      ? tile_worked(ptile)->id
-                      : IDENTITY_NUMBER_ZERO;
-      info.worked16 = info.worked32;
+      info.worked = (NULL != tile_worked(ptile))
+                     ? tile_worked(ptile)->id
+                     : IDENTITY_NUMBER_ZERO;
 
       info.terrain = (NULL != tile_terrain(ptile))
                       ? terrain_number(tile_terrain(ptile))
@@ -554,7 +561,7 @@ void send_tile_info(struct conn_list *dest, struct tile *ptile,
                          : 0;
 
       if (pplayer != NULL) {
-	info.extras = map_get_player_tile(ptile, pplayer)->extras;
+        dbv_to_bv(info.extras.vec, &(map_get_player_tile(ptile, pplayer)->extras));
       } else {
 	info.extras = ptile->extras;
       }
@@ -567,9 +574,9 @@ void send_tile_info(struct conn_list *dest, struct tile *ptile,
       }
 
       send_packet_tile_info(pconn, &info);
-    } else if (pplayer && map_is_known(ptile, pplayer)) {
+    } else if (pplayer != NULL && known) {
       struct player_tile *plrtile = map_get_player_tile(ptile, pplayer);
-      struct vision_site *psite = map_get_player_site(ptile, pplayer);
+      struct vision_site *psite = map_get_playermap_site(plrtile);
 
       info.known = TILE_KNOWN_UNSEEN;
       info.continent = tile_continent(ptile);
@@ -579,10 +586,9 @@ void send_tile_info(struct conn_list *dest, struct tile *ptile,
       eowner = plrtile->extras_owner;
       info.owner = (owner ? player_number(owner) : MAP_TILE_OWNER_NULL);
       info.extras_owner = (eowner ? player_number(eowner) : MAP_TILE_OWNER_NULL);
-      info.worked32 = (NULL != psite)
-                      ? psite->identity
-                      : IDENTITY_NUMBER_ZERO;
-      info.worked16 = info.worked32;
+      info.worked = (NULL != psite)
+                    ? psite->identity
+                    : IDENTITY_NUMBER_ZERO;
 
       info.terrain = (NULL != plrtile->terrain)
                       ? terrain_number(plrtile->terrain)
@@ -593,7 +599,7 @@ void send_tile_info(struct conn_list *dest, struct tile *ptile,
       info.placing = -1;
       info.place_turn = 0;
 
-      info.extras = plrtile->extras;
+      dbv_to_bv(info.extras.vec, &(plrtile->extras));
 
       /* Labels never change, so they are not subject to fog of war */
       if (ptile->label != NULL) {
@@ -608,8 +614,7 @@ void send_tile_info(struct conn_list *dest, struct tile *ptile,
       info.continent = 0;
       info.owner = MAP_TILE_OWNER_NULL;
       info.extras_owner = MAP_TILE_OWNER_NULL;
-      info.worked32 = IDENTITY_NUMBER_ZERO;
-      info.worked16 = IDENTITY_NUMBER_ZERO;
+      info.worked = IDENTITY_NUMBER_ZERO;
 
       info.terrain = terrain_count();
       info.resource = MAX_EXTRA_TYPES;
@@ -648,6 +653,9 @@ void send_map_info(struct conn_list *dest)
   minfo.xsize = wld.map.xsize;
   minfo.ysize = wld.map.ysize;
   minfo.topology_id = wld.map.topology_id;
+  minfo.wrap_id = wld.map.wrap_id;
+  minfo.north_latitude = wld.map.north_latitude;
+  minfo.south_latitude = wld.map.south_latitude;
 
   lsend_packet_map_info(dest, &minfo);
 }
@@ -752,10 +760,10 @@ void map_set_border_vision(struct player *pplayer,
 }
 
 /**********************************************************************//**
-  Shows the area to the player.  Unless the tile is "seen", it will remain
+  Shows the area to the player. Unless the tile is "seen", it will remain
   fogged and units will be hidden.
 
-  Callers may wish to buffer_shared_vision before calling this function.
+  Callers may wish to buffer_shared_vision() before calling this function.
 **************************************************************************/
 void map_show_tile(struct player *src_player, struct tile *ptile)
 {
@@ -807,7 +815,7 @@ void map_show_tile(struct player *src_player, struct tile *ptile)
 /**********************************************************************//**
   Hides the area to the player.
 
-  Callers may wish to buffer_shared_vision before calling this function.
+  Callers may wish to buffer_shared_vision() before calling this function.
 **************************************************************************/
 void map_hide_tile(struct player *src_player, struct tile *ptile)
 {
@@ -880,7 +888,7 @@ void map_show_all(struct player *pplayer)
 }
 
 /**********************************************************************//**
-  Return whether the player knows the tile.  Knowing a tile means you've
+  Return whether the player knows the tile. Knowing a tile means you've
   seen it once (as opposed to seeing a tile which means you can see it now).
 **************************************************************************/
 bool map_is_known(const struct tile *ptile, const struct player *pplayer)
@@ -893,6 +901,18 @@ bool map_is_known(const struct tile *ptile, const struct player *pplayer)
   return dbv_isset(&pplayer->tile_known, tile_index(ptile));
 }
 
+/**********************************************************************//**
+  Returns whether the layer 'vlayer' of the tile 'ptile' is seen
+  by the player 'pplayer'. Only call this when you already know
+  the tile to be known.
+**************************************************************************/
+static inline bool map_is_also_seen(const struct tile *ptile,
+                                    const struct player *pplayer,
+                                    enum vision_layer vlayer)
+{
+  return map_get_seen(pplayer, ptile, vlayer) > 0;
+}
+
 /**********************************************************************//**
   Returns whether the layer 'vlayer' of the tile 'ptile' is known and seen
   by the player 'pplayer'.
@@ -901,16 +921,15 @@ bool map_is_known_and_seen(const struct tile *ptile,
                            const struct player *pplayer,
                            enum vision_layer vlayer)
 {
-  return (map_is_known(ptile, pplayer)
-          && 0 < map_get_seen(pplayer, ptile, vlayer));
+  return map_is_known(ptile, pplayer) && map_is_also_seen(ptile, pplayer, vlayer);
 }
 
 /**********************************************************************//**
-  Return whether the player can see the tile.  Seeing a tile means you have
+  Return whether the player can see the tile. Seeing a tile means you have
   vision of it now (as opposed to knowing a tile which means you've seen it
-  before).  Note that a tile can be seen but not known (currently this only
+  before). Note that a tile can be seen but not known (currently this only
   happens when a city is founded with some unknown tiles in its radius); in
-  this case the tile is unknown (but map_get_seen will still return TRUE).
+  this case the tile is unknown (but map_get_seen() will still return TRUE).
 **************************************************************************/
 static inline int map_get_seen(const struct player *pplayer,
                                const struct tile *ptile,
@@ -1315,7 +1334,7 @@ static void player_tile_init(struct tile *ptile, struct player *pplayer)
   plrtile->owner = NULL;
   plrtile->extras_owner = NULL;
   plrtile->site = NULL;
-  BV_CLR_ALL(plrtile->extras);
+  dbv_init(&(plrtile->extras), extra_count());
   if (!game.server.last_updated_year) {
     plrtile->last_updated = game.info.turn;
   } else {
@@ -1338,6 +1357,8 @@ static void player_tile_free(struct tile *ptile, struct player *pplayer)
   if (plrtile->site != NULL) {
     vision_site_destroy(plrtile->site);
   }
+
+  dbv_free(&(plrtile->extras));
 }
 
 /**********************************************************************//**
@@ -1353,18 +1374,9 @@ struct vision_site *map_get_player_city(const struct tile *ptile,
   return psite;
 }
 
-/**********************************************************************//**
-  Returns site located at given tile from player map.
-**************************************************************************/
-struct vision_site *map_get_player_site(const struct tile *ptile,
-					const struct player *pplayer)
-{
-  return map_get_player_tile(ptile, pplayer)->site;
-}
-
 /**********************************************************************//**
   Players' information of tiles is tracked so that fogged area can be kept
-  consistent even when the client disconnects.  This function returns the
+  consistent even when the client disconnects. This function returns the
   player tile information for the given tile and player.
 **************************************************************************/
 struct player_tile *map_get_player_tile(const struct tile *ptile,
@@ -1388,16 +1400,16 @@ bool update_player_tile_knowledge(struct player *pplayer, struct tile *ptile)
   struct player_tile *plrtile = map_get_player_tile(ptile, pplayer);
 
   if (plrtile->terrain != ptile->terrain
-      || !BV_ARE_EQUAL(plrtile->extras, ptile->extras)
+      || !bv_match_dbv(&(plrtile->extras), ptile->extras.vec)
       || plrtile->resource != ptile->resource
       || plrtile->owner != tile_owner(ptile)
       || plrtile->extras_owner != extra_owner(ptile)) {
     plrtile->terrain = ptile->terrain;
     extra_type_iterate(pextra) {
       if (player_knows_extra_exist(pplayer, pextra, ptile)) {
-	BV_SET(plrtile->extras, extra_number(pextra));
+	dbv_set(&(plrtile->extras), extra_number(pextra));
       } else {
-	BV_CLR(plrtile->extras, extra_number(pextra));
+	dbv_clr(&(plrtile->extras), extra_number(pextra));
       }
     } extra_type_iterate_end;
     if (ptile->resource != NULL
@@ -1474,8 +1486,6 @@ static bool really_give_tile_info_from_player_to_player(struct player *pfrom,
                                                         struct player *pdest,
                                                         struct tile *ptile)
 {
-  struct player_tile *from_tile, *dest_tile;
-
   if (!map_is_known_and_seen(ptile, pdest, V_MAIN)) {
     /* I can just hear people scream as they try to comprehend this if :).
      * Let me try in words:
@@ -1489,12 +1499,14 @@ static bool really_give_tile_info_from_player_to_player(struct player *pfrom,
             && (((map_get_player_tile(ptile, pfrom)->last_updated
                   > map_get_player_tile(ptile, pdest)->last_updated))
                 || !map_is_known(ptile, pdest)))) {
+      struct player_tile *from_tile, *dest_tile;
+
       from_tile = map_get_player_tile(ptile, pfrom);
       dest_tile = map_get_player_tile(ptile, pdest);
       /* Update and send tile knowledge */
       map_set_known(ptile, pdest);
       dest_tile->terrain = from_tile->terrain;
-      dest_tile->extras   = from_tile->extras;
+      dbv_copy(&(dest_tile->extras), &(from_tile->extras));
       dest_tile->resource = from_tile->resource;
       dest_tile->owner    = from_tile->owner;
       dest_tile->extras_owner = from_tile->extras_owner;
@@ -1736,6 +1748,8 @@ void enable_fog_of_war_player(struct player *pplayer)
 {
   const v_radius_t radius_sq = V_RADIUS(-1, 0, 0);
 
+  dlsend_packet_edit_fogofwar_state(pplayer->connections, TRUE);
+
   buffer_shared_vision(pplayer);
   whole_map_iterate(&(wld.map), ptile) {
     map_change_seen(pplayer, ptile, radius_sq, FALSE);
@@ -1760,6 +1774,8 @@ void disable_fog_of_war_player(struct player *pplayer)
 {
   const v_radius_t radius_sq = V_RADIUS(1, 0, 0);
 
+  dlsend_packet_edit_fogofwar_state(pplayer->connections, FALSE);
+
   buffer_shared_vision(pplayer);
   whole_map_iterate(&(wld.map), ptile) {
     map_change_seen(pplayer, ptile, radius_sq, FALSE);
