@@ -392,6 +392,11 @@ struct entry *secfile_insert_filereference(struct section_file *secfile,
                                            const char *path, ...)
                               fc__attribute((__format__ (__printf__, 3, 4)));
 
+struct entry *secfile_insert_comment(struct section_file *secfile,
+                                     const char *str,
+                                     const char *path, ...)
+                              fc__attribute((__format__ (__printf__, 3, 4)));
+
 /* Deletion function. */
 bool secfile_entry_delete(struct section_file *secfile,
                           const char *path, ...)
@@ -590,6 +595,8 @@ secfile_sections(const struct section_file *secfile);
 struct section_list *
 secfile_sections_by_name_prefix(const struct section_file *secfile,
                                 const char *prefix);
+bool secfile_section_prefix_present(const struct section_file *secfile,
+                                    const char *prefix);
 struct section *secfile_section_new(struct section_file *secfile,
                                     const char *section_name);
 
@@ -627,6 +634,7 @@ enum entry_type {
   ENTRY_FLOAT,
   ENTRY_STR,
   ENTRY_FILEREFERENCE,
+  ENTRY_LONG_COMMENT,
   ENTRY_ILLEGAL
 };
 
@@ -661,4 +669,4 @@ bool entry_str_set_gt_marking(struct entry *pentry, bool gt_marking);
 }
 #endif /* __cplusplus */
 
-#endif  /* FC__REGISTRY_INI_H */
+#endif /* FC__REGISTRY_INI_H */
